@@ -43,26 +43,15 @@ from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import transaction
 from django.http import Http404, HttpRequest, HttpResponse, JsonResponse
 from django.urls import reverse
+from django.utils import timezone
 from django.utils.decorators import method_decorator
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic.base import View
 
-from .. import ca_settings
-from ..constants import REASON_CODES
-from ..models import (
-    AcmeAccount,
-    AcmeAuthorization,
-    AcmeCertificate,
-    AcmeChallenge,
-    AcmeOrder,
-    Certificate,
-    CertificateAuthority,
-)
-from ..tasks import acme_issue_certificate, acme_validate_challenge, run_task
-from ..utils import check_name, int_to_hex, make_naive, validate_email
-from .errors import AcmeBadCSR, AcmeException, AcmeForbidden, AcmeMalformed, AcmeUnauthorized
-from .messages import CertificateRequest, NewOrder
-from .responses import (
+from django_ca import ca_settings
+from django_ca.acme.errors import AcmeBadCSR, AcmeException, AcmeForbidden, AcmeMalformed, AcmeUnauthorized
+from django_ca.acme.messages import CertificateRequest, NewOrder
+from django_ca.acme.responses import (
     AcmeResponse,
     AcmeResponseAccount,
     AcmeResponseAccountCreated,
@@ -78,7 +67,19 @@ from .responses import (
     AcmeResponseUnauthorized,
     AcmeResponseUnsupportedMediaType,
 )
-from .utils import parse_acme_csr
+from django_ca.acme.utils import parse_acme_csr
+from django_ca.constants import REASON_CODES
+from django_ca.models import (
+    AcmeAccount,
+    AcmeAuthorization,
+    AcmeCertificate,
+    AcmeChallenge,
+    AcmeOrder,
+    Certificate,
+    CertificateAuthority,
+)
+from django_ca.tasks import acme_issue_certificate, acme_validate_challenge, run_task
+from django_ca.utils import check_name, int_to_hex, validate_email
 
 log = logging.getLogger(__name__)
 MessageTypeVar = TypeVar("MessageTypeVar", bound=jose.json_util.JSONObjectWithFields)
@@ -104,7 +105,7 @@ class ContactValidationMixin:
                 #   Clients MUST NOT provide a "mailto" URL in the "contact" field that contains "hfields"
                 #   [RFC6068] or more than one "addr-spec" in the "to" component.
 
-                # We rule out quoted local address fields, otherwise it's extremely hard to validate
+                # We rule our quoted local address fields, otherwise it's extremely hard to validate
                 # email addresses.
                 if addr.startswith('"'):
                     raise AcmeMalformed("invalidContact", "Quoted local part in email is not allowed.")
@@ -192,7 +193,7 @@ class AcmeDirectory(View):
 class AcmeGetNonceViewMixin:
     """View mixin that provides methods to get and validate a Nonce.
 
-    Note that thix mixin depends on the presence of a ``serial`` argument to the URL resolver.
+    Note that this mixin depends on the presence of a ``serial`` argument to the URL resolver.
     """
 
     kwargs: Dict[str, str]
@@ -268,7 +269,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
         return account.usable
 
     def set_link_relations(self, response: "HttpResponseBase", **kwargs: str) -> None:
-        """Set Link releations headers according to RFC8288.
+        """Set Link relations headers according to RFC8288.
 
         `RFC8555, section 7.1 <https://tools.ietf.org/html/rfc8555#section-7.1>`_ states:
 
@@ -298,12 +299,20 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
     #        with open(prepared_path, 'w') as stream:
     #            json.dump(prepared_data, stream, indent=4)
 
+    # NOINSPECTION/TYPE NOTE: It's okay to be more specific here
+    # noinspection PyMethodOverriding
     def dispatch(  # type: ignore[override]
         self, request: HttpRequest, serial: str, slug: Optional[str] = None
     ) -> "HttpResponseBase":
         if not ca_settings.CA_ENABLE_ACME:
             raise Http404("Page not found.")
 
+        # COVERAGE NOTE: Checking just for safety here.
+        if not isinstance(serial, str):  # pragma: no cover
+            raise ImproperlyConfigured("View expects a str for a serial")
+        if slug is not None and not isinstance(slug, str):  # pragma: no cover
+            raise ImproperlyConfigured("View expects a str for a slug")
+
         try:
             response = super().dispatch(request, serial=serial, slug=slug)
         except AcmeException as ex:
@@ -334,7 +343,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
             # "Because client requests in ACME carry JWS objects in the Flattened JSON Serialization, they
             # must have the Content-Type header field set to "application/jose+json".  If a request does not
             # meet this requirement, then the server MUST return a response with status code 415 (Unsupported
-            # Media Type).
+            # Media Type)."
             return AcmeResponseUnsupportedMediaType()
 
         # self.prepared['body'] = json.loads(request.body.decode('utf-8'))
@@ -392,7 +401,7 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
             # RFC 8555, 6.2: "The JWS MUST NOT have multiple signatures"
             return AcmeResponseMalformed(message="Multiple JWS signatures encountered.")
 
-        # "The JWS Protected Header MUST include the following fields:...
+        # "The JWS Protected Header MUST include the following fields:..."
         if not combined.alg:  # pragma: no cover
             # ... "alg"
             return AcmeResponseMalformed(message="No algorithm specified.")
@@ -475,10 +484,16 @@ class AcmeNewNonceView(AcmeGetNonceViewMixin, View):
        * `RFC 8555, section 7.2 <https://tools.ietf.org/html/rfc8555#section-7.2>`_
     """
 
+    # NOINSPECTION/TYPE NOTE: It's okay to be more specific here
+    # noinspection PyMethodOverriding
     def dispatch(self, request: HttpRequest, serial: str) -> "HttpResponseBase":  # type: ignore[override]
         if not ca_settings.CA_ENABLE_ACME:
             raise Http404("Page not found.")
 
+        # COVERAGE NOTE: Checking just for safety here.
+        if not isinstance(serial, str):  # pragma: no cover
+            raise ImproperlyConfigured("View expects a str for a serial")
+
         response = super().dispatch(request, serial)
         response["replay-nonce"] = self.get_nonce()
 
@@ -684,9 +699,11 @@ class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
         now = datetime.now(tz.utc)
 
         # josepy message classes define field names as class variables, but instance attributes are of the
-        # same type (similar to Django). So we cast fields detected as RFC3339Field to datetime.
+        # same type (similar to Django). So we cast josepy fields to the actual type.
         not_before = cast(Optional[datetime], message.not_before)
         not_after = cast(Optional[datetime], message.not_after)
+        # TODO: test if identifiers are acceptable
+        identifiers = typing.cast(List[messages.Identifier], message.identifiers)
 
         if not_before and not_before < now:
             raise AcmeMalformed(message="Certificate cannot be valid before now.")
@@ -694,22 +711,19 @@ class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
             raise AcmeMalformed(message="Certificate cannot be valid that long.")
         if not_before and not_after and not_before > not_after:
             raise AcmeMalformed(message="notBefore must be before notAfter.")
-        if not message.identifiers:
+        if not identifiers:
             # NOTE: Catches sending an empty tuple, which is not caught in message deserialization
             raise AcmeMalformed(message="The following fields are required: identifiers")
 
         if settings.USE_TZ is False:
-            if not_before is not None:
-                not_before = make_naive(not_before)
-            if not_after is not None:
-                not_after = make_naive(not_after)
+            if not_before is not None and timezone.is_aware(not_before):
+                not_before = timezone.make_naive(not_before)
+            if not_after is not None and timezone.is_aware(not_after):
+                not_after = timezone.make_naive(not_after)
 
-        # TODO: test if identifiers are acceptable
         order = AcmeOrder.objects.create(account=self.account, not_before=not_before, not_after=not_after)
-
         authorizations = [
-            self.request.build_absolute_uri(authz.acme_url)
-            for authz in order.add_authorizations(message.identifiers)
+            self.request.build_absolute_uri(authz.acme_url) for authz in order.add_authorizations(identifiers)
         ]
 
         expires = order.expires
@@ -720,7 +734,7 @@ class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
             authorizations=authorizations,
             expires=expires,
             finalize=self.request.build_absolute_uri(order.acme_finalize_url),
-            identifiers=message.identifiers,
+            identifiers=identifiers,
             not_after=message.not_after,
             not_before=message.not_before,
             status=order.status,
@@ -814,20 +828,16 @@ class AcmeOrderFinalizeView(AcmeMessageBaseView[CertificateRequest]):
             log.exception("Error parsing CSR.")
             raise AcmeBadCSR(message="Unable to parse CSR.") from ex
 
-        if csr.is_signature_valid is False:
-            raise AcmeBadCSR(message="CSR signature is not valid.")
-
         # Do not accept MD5 or SHA1 signatures
         hash_algorithm = csr.signature_hash_algorithm
-        if hasattr(hashes, "MD5") and isinstance(
-            hash_algorithm, hashes.MD5
-        ):  # pragma: only cryptography<39.0
+        if hasattr(hashes, "MD5") and isinstance(hash_algorithm, hashes.MD5):
             raise AcmeBadCSR(message=f"{hash_algorithm.name}: Insecure hash algorithm.")
-        if hasattr(hashes, "SHA1") and isinstance(  # pragma: only cryptography<39.0
-            hash_algorithm, hashes.SHA1
-        ):
+        if hasattr(hashes, "SHA1") and isinstance(hash_algorithm, hashes.SHA1):
             raise AcmeBadCSR(message=f"{hash_algorithm.name}: Insecure hash algorithm.")
 
+        if csr.is_signature_valid is False:
+            raise AcmeBadCSR(message="CSR signature is not valid.")
+
         # Get list of general names from the authorizations
         names_from_order = set(auth.general_name for auth in authorizations)
 
@@ -935,6 +945,11 @@ class AcmeCertificateView(AcmePostAsGetView):
         except AcmeCertificate.DoesNotExist as ex:
             raise AcmeUnauthorized() from ex
 
+        # COVERAGE NOTE: This should not happen in practice, as viewable() above already excludes those
+        # instances. We add this check here just to be sure (and to make mypy happy).
+        if cert.cert is None:  # pragma: no cover
+            raise AcmeUnauthorized()
+
         # self.prepared['cert'] = slug
         # self.prepared['csr'] = cert.csr
         # self.prepared['order'] = cert.order.slug
@@ -1083,7 +1098,7 @@ class AcmeCertificateRevocationView(AcmeMessageBaseView[messages.Revocation]):
         certs = Certificate.objects.filter(ca=self.ca).currently_valid()
 
         # If the request is signed with the certificate key (and not the account), a JWK is set for this
-        # request and we verify it was signed by the certificate on record.
+        # request, and we verify it was signed by the certificate on record.
         if self.jws.signature.combined.jwk:
             # NOTE: RFC 8555, section 7.6 states:
             #
@@ -1093,7 +1108,7 @@ class AcmeCertificateRevocationView(AcmeMessageBaseView[messages.Revocation]):
             # This implies that the account used to request the certificate may even be revoked or invalid, as
             # long as the private key is used to sign the request. So we don't look at the ACME account at all
             # here.
-            cert = certs.get(serial=serial)
+            cert: Certificate = certs.get(serial=serial)
 
             jwk = cert.jwk
 
@@ -1117,7 +1132,7 @@ class AcmeCertificateRevocationView(AcmeMessageBaseView[messages.Revocation]):
             if cert.acmecertificate.order.account == self.account:
                 return cert
 
-            # If the account holds authorizations for all of the identifiers in the certificate, it can also
+            # If the account holds authorizations for all the identifiers in the certificate, it can also
             # be revoked, so get a list of all currently valid authorizations that the account holds
             authz = set(AcmeAuthorization.objects.dns().valid().account(account=self.account).names())
 
