@@ -1,29 +1,62 @@
 # This file is part of django-ca (https://github.com/mathiasertl/django-ca).
 #
-# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
-# General Public License as published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 #
-# You should have received a copy of the GNU General Public License along with django-ca.  If not,
-# see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with django-ca. If not, see
+# <http://www.gnu.org/licenses/>.
 
 """Collection of constants used by django-ca."""
 
 import enum
+import typing
 from collections import defaultdict
 from types import MappingProxyType
+from typing import Type
 
 from cryptography import x509
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.asymmetric import dsa, ec, ed448, ed25519, rsa
 from cryptography.x509.certificate_transparency import LogEntryType
 from cryptography.x509.oid import ExtendedKeyUsageOID as _ExtendedKeyUsageOID
-from cryptography.x509.oid import ExtensionOID
+from cryptography.x509.oid import ExtensionOID, NameOID
 
 from django.utils.translation import gettext_lazy as _
 
+# IMPORTANT: Do **not** import any module from django_ca at runtime here, or you risk circular imports.
+if typing.TYPE_CHECKING:
+    from django_ca.typehints import AllowedHashTypes
+
+#: Mapping of elliptic curve names to the implementing classes
+ELLIPTIC_CURVE_TYPES: "MappingProxyType[str, Type[ec.EllipticCurve]]" = MappingProxyType(
+    {
+        ec.SECT571R1.name: ec.SECT571R1,
+        ec.SECT409R1.name: ec.SECT409R1,
+        ec.SECT283R1.name: ec.SECT283R1,
+        ec.SECT233R1.name: ec.SECT233R1,
+        ec.SECT163R2.name: ec.SECT163R2,
+        ec.SECT571K1.name: ec.SECT571K1,
+        ec.SECT409K1.name: ec.SECT409K1,
+        ec.SECT283K1.name: ec.SECT283K1,
+        ec.SECT233K1.name: ec.SECT233K1,
+        ec.SECT163K1.name: ec.SECT163K1,
+        ec.SECP521R1.name: ec.SECP521R1,
+        ec.SECP384R1.name: ec.SECP384R1,
+        ec.SECP256R1.name: ec.SECP256R1,
+        ec.SECP256K1.name: ec.SECP256K1,
+        ec.SECP224R1.name: ec.SECP224R1,
+        ec.SECP192R1.name: ec.SECP192R1,
+        ec.BrainpoolP256R1.name: ec.BrainpoolP256R1,
+        ec.BrainpoolP384R1.name: ec.BrainpoolP384R1,
+        ec.BrainpoolP512R1.name: ec.BrainpoolP512R1,
+    }
+)
+
 
 class ExtendedKeyUsageOID(_ExtendedKeyUsageOID):
     """Extend the OIDs known to cryptography with what users needed over the years."""
@@ -33,10 +66,7 @@ class ExtendedKeyUsageOID(_ExtendedKeyUsageOID):
     IPSEC_TUNNEL = x509.ObjectIdentifier("1.3.6.1.5.5.7.3.6")
     IPSEC_USER = x509.ObjectIdentifier("1.3.6.1.5.5.7.3.7")
 
-    # Used by PKINIT logon on Windows (see  github #46)
-    SMARTCARD_LOGON = x509.ObjectIdentifier("1.3.6.1.4.1.311.20.2.2")
-
-    # mobilee Driving Licence or mDL (see ISO/IEC DIS 18013-5, GitHub PR #81)
+    # mobile Driving Licence or mDL (see ISO/IEC DIS 18013-5, GitHub PR #81)
     MDL_DOCUMENT_SIGNER = x509.ObjectIdentifier("1.0.18013.5.1.2")
     MDL_JWS_CERTIFICATE = x509.ObjectIdentifier("1.0.18013.5.1.3")
 
@@ -45,10 +75,6 @@ class ExtendedKeyUsageOID(_ExtendedKeyUsageOID):
     if not hasattr(_ExtendedKeyUsageOID, "CERTIFICATE_TRANSPARENCY"):  # pragma: cryptography<38.0 branch
         CERTIFICATE_TRANSPARENCY = x509.ObjectIdentifier("1.3.6.1.4.1.11129.2.4.4")
 
-    # ExtendedKeyUsageOID.IPSEC_IKE was added in cryptography==37.0.0.
-    if not hasattr(_ExtendedKeyUsageOID, "IPSEC_IKE"):  # pragma: cryptography<37.0 branch
-        IPSEC_IKE = x509.ObjectIdentifier("1.3.6.1.5.5.7.3.17")
-
 
 #: Map of ExtendedKeyUsageOIDs to names in RFC 5280 (and other RFCs).
 EXTENDED_KEY_USAGE_NAMES = MappingProxyType(
@@ -72,7 +98,10 @@ EXTENDED_KEY_USAGE_NAMES = MappingProxyType(
     }
 )
 
-#: Map of ExtendedKeyUsageOIDs to human readable names.
+#: Map of Extended Key Usage names to ExtendedKeyUsageOID (the inverse of EXTENDED_KEY_USAGE_NAMES).
+EXTENDED_KEY_USAGE_OIDS = MappingProxyType({v: k for k, v in EXTENDED_KEY_USAGE_NAMES.items()})
+
+#: Map of ExtendedKeyUsageOIDs to human-readable names.
 EXTENDED_KEY_USAGE_HUMAN_READABLE_NAMES = MappingProxyType(
     {
         ExtendedKeyUsageOID.ANY_EXTENDED_KEY_USAGE: "Any Extended Key Usage",
@@ -157,7 +186,7 @@ EXTENSION_DEFAULT_CRITICAL = MappingProxyType(
         ExtensionOID.SUBJECT_DIRECTORY_ATTRIBUTES: False,  # MUST mark this extension as non-critical
         ExtensionOID.SUBJECT_INFORMATION_ACCESS: False,  # MUST mark this extension as non-critical
         ExtensionOID.SUBJECT_KEY_IDENTIFIER: False,  # MUST mark this extension as non-critical
-        ExtensionOID.TLS_FEATURE: False,  # RFC 7633: MUST NOT be marked critical
+        ExtensionOID.TLS_FEATURE: False,  # RFC 7633: SHOULD NOT be marked critical
     }
 )
 
@@ -195,7 +224,7 @@ EXTENSION_KEYS = MappingProxyType(
 #: Map of extension keys to ExtensionOIDs (the inverse of EXTENSION_KEYS).
 EXTENSION_KEY_OIDS = MappingProxyType({v: k for k, v in EXTENSION_KEYS.items()})
 
-#: Map of ExtensionOIDs to human readable names as they appear in the RFC where they are defined.
+#: Map of ExtensionOIDs to human-readable names as they appear in the RFC where they are defined.
 EXTENSION_NAMES = MappingProxyType(
     {
         ExtensionOID.AUTHORITY_INFORMATION_ACCESS: "Authority Information Access",
@@ -240,6 +269,52 @@ EXTENSION_RFC_DEFINITION = MappingProxyType(
     )
 )
 
+# pragma: only django-ca<1.27  # support for non-standard hash algorithm names is dropped in 1.27
+HASH_ALGORITHM_KEYS: "MappingProxyType[Type[AllowedHashTypes], str]" = MappingProxyType(
+    {
+        hashes.SHA224: hashes.SHA224.name,
+        hashes.SHA256: hashes.SHA256.name,
+        hashes.SHA384: hashes.SHA384.name,
+        hashes.SHA512: hashes.SHA512.name,
+        hashes.SHA3_224: hashes.SHA3_224.name,
+        hashes.SHA3_256: hashes.SHA3_256.name,
+        hashes.SHA3_384: hashes.SHA3_384.name,
+        hashes.SHA3_512: hashes.SHA3_512.name,
+    }
+)
+
+# pragma: only django-ca<1.27  # support for non-standard hash algorithm names is dropped in 1.27
+HASH_ALGORITHM_KEY_TYPES: "MappingProxyType[str, Type[AllowedHashTypes]]" = MappingProxyType(
+    {v: k for k, v in HASH_ALGORITHM_KEYS.items()}
+)
+
+# Map of hash algorithm types in cryptography to standard hash algorithm names. The values can be used for
+# ``--algorithm`` command line parameter.
+HASH_ALGORITHM_NAMES: "MappingProxyType[Type[AllowedHashTypes], str]" = MappingProxyType(
+    {
+        hashes.SHA224: "SHA-224",
+        hashes.SHA256: "SHA-256",
+        hashes.SHA384: "SHA-384",
+        hashes.SHA512: "SHA-512",
+        hashes.SHA3_224: "SHA3/224",
+        hashes.SHA3_256: "SHA3/256",
+        hashes.SHA3_384: "SHA3/384",
+        hashes.SHA3_512: "SHA3/512",
+        # cryptography does not support these hash algorithms for signatures
+        # hashes.SHA512_224: "SHA-512/224",
+        # hashes.SHA512_256: "SHA-512/256",
+        # hashes.SM3: "SM3",
+        # hashes.SHAKE128: "SHAKE128",
+        # hashes.SHAKE256: "SHAKE256",
+        # hashes.BLAKE2b: "BLAKE2b",  # https://datatracker.ietf.org/doc/html/rfc7693.html
+        # hashes.BLAKE2s: "BLAKE2s",  # https://datatracker.ietf.org/doc/html/rfc7693.html
+    }
+)
+
+#: Mapping of hash algorithm names to hash algorithm types (the inverse of HASH_ALGORITHM_NAMES).
+HASH_ALGORITHM_TYPES: "MappingProxyType[str, Type[AllowedHashTypes]]" = MappingProxyType(
+    {v: k for k, v in HASH_ALGORITHM_NAMES.items()}
+)
 
 #: Map of `kwargs` for :py:class:`~cg:cryptography.x509.KeyUsage` to names in RFC 5280.
 KEY_USAGE_NAMES = MappingProxyType(
@@ -265,12 +340,87 @@ LOG_ENTRY_TYPE_KEYS = MappingProxyType(
     }
 )
 
-#: Map of human readable names/serialized values to TLSFeatureTypes.
+# Human-readable names come from RFC 4519 except where noted
+#: Map OID objects to IDs used in subject strings
+NAME_OID_NAMES = MappingProxyType(
+    {
+        **{
+            NameOID.BUSINESS_CATEGORY: "businessCategory",
+            NameOID.COMMON_NAME: "CN",
+            NameOID.COUNTRY_NAME: "C",
+            NameOID.DN_QUALIFIER: "dnQualifier",
+            NameOID.DOMAIN_COMPONENT: "DC",
+            NameOID.EMAIL_ADDRESS: "emailAddress",  # not specified in RFC 4519
+            NameOID.GENERATION_QUALIFIER: "generationQualifier",
+            NameOID.GIVEN_NAME: "givenName",
+            NameOID.INN: "inn",  # undocumented
+            NameOID.JURISDICTION_COUNTRY_NAME: "jurisdictionCountryName",
+            NameOID.JURISDICTION_LOCALITY_NAME: "jurisdictionLocalityName",
+            NameOID.JURISDICTION_STATE_OR_PROVINCE_NAME: "jurisdictionStateOrProvinceName",
+            NameOID.LOCALITY_NAME: "L",
+            NameOID.OGRN: "ogrn",  # undocumented
+            NameOID.ORGANIZATIONAL_UNIT_NAME: "OU",
+            NameOID.ORGANIZATION_NAME: "O",
+            NameOID.POSTAL_ADDRESS: "postalAddress",
+            NameOID.POSTAL_CODE: "postalCode",
+            NameOID.PSEUDONYM: "pseudonym",  # not specified in RFC 4519
+            NameOID.SERIAL_NUMBER: "serialNumber",
+            NameOID.SNILS: "snils",  # undocumented
+            NameOID.STATE_OR_PROVINCE_NAME: "ST",
+            NameOID.STREET_ADDRESS: "street",
+            NameOID.SURNAME: "sn",
+            NameOID.TITLE: "title",
+            NameOID.UNSTRUCTURED_NAME: "unstructuredName",  # not specified in RFC 4519
+            NameOID.USER_ID: "uid",
+            NameOID.X500_UNIQUE_IDENTIFIER: "x500UniqueIdentifier",
+        },
+        **({NameOID.INITIALS: "initials"} if hasattr(NameOID, "INITIALS") else {}),  # added in cg==41
+    }
+)
+
+#: Map NameOID names to cryptography NameOID objects. This variant adds all RFC 4519 aliases as well.
+NAME_OID_TYPES = MappingProxyType(
+    {
+        **{v: k for k, v in NAME_OID_NAMES.items()},
+        **{
+            "commonName": NameOID.COMMON_NAME,
+            "countryName": NameOID.COUNTRY_NAME,
+            "domainComponent": NameOID.DOMAIN_COMPONENT,
+            "localityName": NameOID.LOCALITY_NAME,
+            "organizationName": NameOID.ORGANIZATION_NAME,
+            "organizationalUnitName": NameOID.ORGANIZATIONAL_UNIT_NAME,
+            "streetAddress": NameOID.STREET_ADDRESS,  # not specified in RFC 4519, but consistent with others
+            "surname": NameOID.SURNAME,
+            "userid": NameOID.USER_ID,
+        },
+    }
+)
+
+#: Tuple of supported public key types.
+PUBLIC_KEY_TYPES = (
+    dsa.DSAPublicKey,
+    ec.EllipticCurvePublicKey,
+    ed25519.Ed25519PublicKey,
+    ed448.Ed448PublicKey,
+    rsa.RSAPublicKey,
+)
+
+#: Tuple of supported private key types.
+PRIVATE_KEY_TYPES = (
+    dsa.DSAPrivateKey,
+    ec.EllipticCurvePrivateKey,
+    ed25519.Ed25519PrivateKey,
+    ed448.Ed448PrivateKey,
+    rsa.RSAPrivateKey,
+)
+PARSABLE_KEY_TYPES = ("RSA", "DSA", "EC", "Ed25519", "Ed448")
+
+#: Map of human-readable names/serialized values to TLSFeatureTypes.
 TLS_FEATURE_NAMES = MappingProxyType(
     {
         # https://tools.ietf.org/html/rfc6066.html:
-        "OCSPMustStaple": x509.TLSFeatureType.status_request,
         "status_request": x509.TLSFeatureType.status_request,
+        "OCSPMustStaple": x509.TLSFeatureType.status_request,
         # https://tools.ietf.org/html/rfc6961.html (not commonly used):
         "MultipleCertStatusRequest": x509.TLSFeatureType.status_request_v2,
         "status_request_v2": x509.TLSFeatureType.status_request_v2,
