@@ -1,15 +1,15 @@
 # This file is part of django-ca (https://github.com/mathiasertl/django-ca).
 #
-# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
-# General Public License as published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 #
-# You should have received a copy of the GNU General Public License along with django-ca. If not,
-# see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with django-ca. If not, see
+# <http://www.gnu.org/licenses/>.
 
 """Django model managers."""
 
@@ -17,43 +17,47 @@
 import pathlib
 import typing
 import warnings
-from typing import TYPE_CHECKING, Any, Generic, Iterable, List, Optional, Sequence, Tuple, TypeVar, Union
+from typing import Any, Dict, Generic, Iterable, List, Optional, Sequence, Tuple, TypeVar, Union
 
 from cryptography import x509
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import ec
-from cryptography.hazmat.primitives.hashes import HashAlgorithm
 from cryptography.hazmat.primitives.serialization import Encoding, PrivateFormat
-from cryptography.x509.oid import AuthorityInformationAccessOID
+from cryptography.x509.oid import AuthorityInformationAccessOID, ExtensionOID
 
 from django.core.files.base import ContentFile
 from django.db import models
 from django.urls import reverse
 
-from . import ca_settings
-from .constants import EXTENSION_DEFAULT_CRITICAL
-from .deprecation import RemovedInDjangoCA123Warning, deprecate_argument, deprecate_type
-from .extensions import Extension, IssuerAlternativeName, NameConstraints
-from .modelfields import LazyCertificateSigningRequest
-from .openssh import SshHostCaExtension, SshUserCaExtension
-from .profiles import Profile, profiles
-from .signals import post_create_ca, post_issue_cert, pre_create_ca
-from .typehints import Expires, ParsableExtension, ParsableKeyType, X509CertMixinTypeVar
-from .utils import (
+from django_ca import ca_settings, constants
+from django_ca.deprecation import RemovedInDjangoCA126Warning, deprecate_argument, deprecate_type
+from django_ca.extensions.utils import format_extensions, get_formatting_context
+from django_ca.modelfields import LazyCertificateSigningRequest
+from django_ca.openssh import SshHostCaExtension, SshUserCaExtension
+from django_ca.profiles import Profile, profiles
+from django_ca.signals import post_create_ca, post_issue_cert, pre_create_ca
+from django_ca.typehints import (
+    AllowedHashTypes,
+    Expires,
+    ExtensionMapping,
+    ParsableKeyType,
+    X509CertMixinTypeVar,
+)
+from django_ca.utils import (
     ca_storage,
     format_general_name,
     generate_private_key,
     get_cert_builder,
-    int_to_hex,
     parse_expires,
     parse_general_name,
     validate_hostname,
-    validate_key_parameters,
+    validate_private_key_parameters,
+    validate_public_key_parameters,
 )
 
 # https://mypy.readthedocs.io/en/latest/runtime_troubles.html
-if TYPE_CHECKING:
-    from .models import (
+if typing.TYPE_CHECKING:
+    from django_ca.models import (
         AcmeAccount,
         AcmeAuthorization,
         AcmeCertificate,
@@ -62,7 +66,7 @@ if TYPE_CHECKING:
         Certificate,
         CertificateAuthority,
     )
-    from .querysets import AcmeAccountQuerySet, CertificateAuthorityQuerySet, CertificateQuerySet
+    from django_ca.querysets import AcmeAccountQuerySet, CertificateAuthorityQuerySet, CertificateQuerySet
 
     AcmeAccountManagerBase = models.Manager[AcmeAccount]
     AcmeAuthorizationManagerBase = models.Manager[AcmeAuthorization]
@@ -87,9 +91,9 @@ else:
 class CertificateManagerMixin(Generic[X509CertMixinTypeVar, QuerySetTypeVar]):
     """Mixin for model managers."""
 
-    if TYPE_CHECKING:
+    if typing.TYPE_CHECKING:
         # django-stubs (mypy plugin for Django) currently typehints queryset methods as returning a manager,
-        # and does not know about queryset methods comming from the queryset. We typehint basic queryset
+        # and does not know about queryset methods coming from the queryset. We typehint basic queryset
         # methods here, so that mypy knows that returned objects are querysets.
         #
         # The type overrides are because of the return type, as mypy thinks they should return a manager.
@@ -118,10 +122,7 @@ class CertificateManagerMixin(Generic[X509CertMixinTypeVar, QuerySetTypeVar]):
             ...
 
     def get_common_extensions(
-        self,
-        issuer_url: Optional[str] = None,
-        crl_url: Optional[Iterable[str]] = None,
-        ocsp_url: Optional[str] = None,
+        self, crl_url: Optional[Iterable[str]] = None
     ) -> List[Tuple[bool, Union[x509.CRLDistributionPoints, x509.AuthorityInformationAccess]]]:
         """Add extensions potentially common to both CAs and certs."""
 
@@ -133,38 +134,8 @@ class CertificateManagerMixin(Generic[X509CertMixinTypeVar, QuerySetTypeVar]):
                 for c in urls
             ]
             extensions.append((False, x509.CRLDistributionPoints(dps)))
-        auth_info_access = []
-        if ocsp_url:
-            uri = x509.UniformResourceIdentifier(ocsp_url)
-            auth_info_access.append(
-                x509.AccessDescription(access_method=AuthorityInformationAccessOID.OCSP, access_location=uri)
-            )
-        if issuer_url:
-            uri = x509.UniformResourceIdentifier(issuer_url)
-            auth_info_access.append(
-                x509.AccessDescription(
-                    access_method=AuthorityInformationAccessOID.CA_ISSUERS, access_location=uri
-                )
-            )
-        if auth_info_access:
-            extensions.append((False, x509.AuthorityInformationAccess(auth_info_access)))
-        return extensions
 
-    def _extra_extensions(
-        self,
-        builder: x509.CertificateBuilder,
-        extra_extensions: typing.List[typing.Union[x509.Extension[x509.ExtensionType]]],
-    ) -> x509.CertificateBuilder:
-        warn = "Passing a django_ca.extensions.Extension is deprecated and will be removed in django_ca 1.23."
-        for ext in extra_extensions:
-            if isinstance(ext, x509.Extension):
-                builder = builder.add_extension(ext.value, critical=ext.critical)
-            elif isinstance(ext, Extension):
-                warnings.warn(warn, category=RemovedInDjangoCA123Warning, stacklevel=2)
-                builder = builder.add_extension(*ext.for_builder())
-            else:
-                raise ValueError(f"Cannot add extension of type {type(ext).__name__}")
-        return builder
+        return extensions
 
 
 class CertificateAuthorityManager(
@@ -173,7 +144,7 @@ class CertificateAuthorityManager(
 ):
     """Model manager for the CertificateAuthority model."""
 
-    if TYPE_CHECKING:
+    if typing.TYPE_CHECKING:
         # See CertificateManagerMixin for description on this branch
         #
         # pylint: disable=missing-function-docstring; just defining stubs here
@@ -196,54 +167,118 @@ class CertificateAuthorityManager(
         def usable(self) -> "CertificateAuthorityQuerySet":
             ...
 
-    @deprecate_argument("name_constraints", RemovedInDjangoCA123Warning)
-    @deprecate_type("issuer_alt_name", (str, IssuerAlternativeName), RemovedInDjangoCA123Warning)
+    def _get_formatting_context(self, serial: int, signer_serial: int) -> Dict[str, Union[int, str]]:
+        context = get_formatting_context(serial, signer_serial)
+        kwargs = {"serial": context["SIGNER_SERIAL_HEX"]}
+        context["OCSP_PATH"] = reverse("django_ca:ocsp-ca-post", kwargs=kwargs).lstrip("/")
+        context["CRL_PATH"] = reverse("django_ca:ca-crl", kwargs=kwargs).lstrip("/")
+        return context
+
+    def _handle_authority_information_access(
+        self,
+        extensions: ExtensionMapping,
+        ca_issuer_url: Optional[Sequence[str]],
+        ca_ocsp_url: Optional[Sequence[str]],
+    ) -> None:
+        access_descriptions: List[x509.AccessDescription] = []
+        if ExtensionOID.AUTHORITY_INFORMATION_ACCESS in extensions:
+            extension = typing.cast(
+                x509.AuthorityInformationAccess, extensions[ExtensionOID.AUTHORITY_INFORMATION_ACCESS].value
+            )
+            access_descriptions = list(extension)
+
+        has_ocsp = any(ad.access_method == AuthorityInformationAccessOID.OCSP for ad in access_descriptions)
+        has_issuer = any(
+            ad.access_method == AuthorityInformationAccessOID.CA_ISSUERS for ad in access_descriptions
+        )
+
+        if ca_issuer_url and has_issuer is False:
+            access_descriptions += [
+                x509.AccessDescription(
+                    access_method=AuthorityInformationAccessOID.CA_ISSUERS,
+                    access_location=parse_general_name(name),
+                )
+                for name in ca_issuer_url
+            ]
+        if ca_ocsp_url and has_ocsp is False:
+            access_descriptions += [
+                x509.AccessDescription(
+                    access_method=AuthorityInformationAccessOID.OCSP, access_location=parse_general_name(name)
+                )
+                for name in ca_ocsp_url
+            ]
+
+        # Finally sort by OID so that we have more predictable behavior
+        access_descriptions = sorted(access_descriptions, key=lambda ad: ad.access_method.dotted_string)
+
+        if access_descriptions:
+            extensions[ExtensionOID.AUTHORITY_INFORMATION_ACCESS] = x509.Extension(
+                oid=ExtensionOID.AUTHORITY_INFORMATION_ACCESS,
+                critical=False,
+                value=x509.AuthorityInformationAccess(access_descriptions),
+            )
+
+    @deprecate_argument("permitted_subtrees", RemovedInDjangoCA126Warning)
+    @deprecate_argument("excluded_subtrees", RemovedInDjangoCA126Warning)
+    @deprecate_argument("ca_issuer_url", RemovedInDjangoCA126Warning)
+    @deprecate_argument("ca_crl_url", RemovedInDjangoCA126Warning)
+    @deprecate_argument("ca_ocsp_url", RemovedInDjangoCA126Warning)
+    @deprecate_type("ca_ocsp_url", str, RemovedInDjangoCA126Warning)
+    @deprecate_type("ca_issuer_url", str, RemovedInDjangoCA126Warning)
     def init(
         self,
         name: str,
         subject: x509.Name,
         expires: Expires = None,
-        algorithm: typing.Optional[HashAlgorithm] = None,
+        algorithm: Optional[AllowedHashTypes] = None,
         parent: Optional["CertificateAuthority"] = None,
         default_hostname: Optional[Union[bool, str]] = None,
-        pathlen: Optional[int] = None,
+        path_length: Optional[int] = None,
         issuer_url: Optional[str] = None,
-        issuer_alt_name: typing.Optional[x509.Extension[x509.IssuerAlternativeName]] = None,
+        issuer_alt_name: Optional[x509.Extension[x509.IssuerAlternativeName]] = None,
         crl_url: Optional[Iterable[str]] = None,
         ocsp_url: Optional[str] = None,
-        ca_issuer_url: Optional[str] = None,
+        ca_issuer_url: Optional[Sequence[str]] = None,
         ca_crl_url: Optional[Sequence[str]] = None,
-        ca_ocsp_url: Optional[str] = None,
-        name_constraints: Optional[Union[ParsableExtension, NameConstraints]] = None,
-        permitted_subtrees: typing.Optional[typing.Iterable[x509.GeneralName]] = None,
-        excluded_subtrees: typing.Optional[typing.Iterable[x509.GeneralName]] = None,
+        ca_ocsp_url: Optional[Sequence[str]] = None,
+        permitted_subtrees: Optional[Iterable[x509.GeneralName]] = None,
+        excluded_subtrees: Optional[Iterable[x509.GeneralName]] = None,
         password: Optional[Union[str, bytes]] = None,
         parent_password: Optional[Union[str, bytes]] = None,
-        ecc_curve: Optional[ec.EllipticCurve] = None,
+        elliptic_curve: Optional[ec.EllipticCurve] = None,
         key_type: ParsableKeyType = "RSA",
         key_size: Optional[int] = None,
-        extra_extensions: Optional[typing.Iterable[x509.Extension[x509.ExtensionType]]] = None,
+        extensions: Optional[Iterable[x509.Extension[x509.ExtensionType]]] = None,
         path: Union[pathlib.PurePath, str] = "ca",
         caa: str = "",
         website: str = "",
         terms_of_service: str = "",
         acme_enabled: bool = False,
         acme_requires_contact: bool = True,
+        acme_profile: Optional[str] = None,
         openssh_ca: bool = False,
     ) -> "CertificateAuthority":
         """Create a new certificate authority.
 
-        .. deprecated:: 1.21.0
+        .. versionchanged:: 1.23.0
+
+           * The `ecc_curve` parameter was renamed to `elliptic_curve`.
+           * Passing ``key_type="EdDSA"`` is deprecated, use ``key_type="Ed25519"`` instead.
+           * Passing ``key_type="ECC"`` is deprecated, use ``key_type="EC"`` instead.
+
+        .. versionchanged:: 1.24.0
+
+           * The `extra_extensions` parameter was renamed to `extensions`.
+           * The `pathlen` parameter was renamed to `path_length`.
+           * The `ca_issuer_url` and `ca_ocsp_url` parameters are now list of strings. Support for bare
+             strings will be removed in ``django-ca==1.26.0``.
+
+        .. versionchanged:: 1.25.0
+
+           * The `permitted_subtrees` and `excluded_subtrees` subtrees are deprecated and will be removed in
+             ``django-ca==1.26.0``. Pass a :py:class:`~cg:cryptography.x509.NameConstraints` extension in
+             `extensions` instead.
 
-           * The `name_constraints` parameter is deprecated and will be removed in ``django_ca==1.23``. Use
-             the `permitted_subtrees` and `excluded_subtrees` parameter instead.
-           * Passing  ``django_ca.extensions.Extension`` instance to `extra_extensions` is now deprecated. The
-             feature will be removed in ``django_ca==1.23``. Pass a ``x509.Extension`` instance instead.
-           * The `issuer_alt_name` now accepts a
-             :py:class:`~cg:cryptography.x509.Extension` with a
-             :py:class:`~cg:cryptography.x509.IssuerAlternativeName` value, passing a `str` or
-             ``django_ca.extensions.IssuerAlternativeName`` is deprecated and will be removed in
-             ``django_ca==1.23``.
 
         Parameters
         ----------
@@ -257,14 +292,16 @@ class CertificateAuthorityManager(
             <settings-ca-default-expires>`.
         algorithm : :py:class:`~cg:cryptography.hazmat.primitives.hashes.HashAlgorithm`, optional
             Hash algorithm used when signing the certificate, defaults to
-            :ref:`CA_DIGEST_ALGORITHM <settings-ca-digest-algorithm>`.
+            :ref:`settings-ca-default-signature-hash-algorithm` for RSA/EC keys, and
+            :ref:`settings-ca-default-dsa-signature-hash-algorithm` for DSA keys. Passing an algorithm for
+            Ed448/Ed25519 keys is an error.
         parent : :py:class:`~django_ca.models.CertificateAuthority`, optional
             Parent certificate authority for the new CA. Passing this value makes the CA an intermediate
             authority. Let unset if this CA will be used for OpenSSH.
         default_hostname : str, optional
-            Override the URLconfigured with :ref:`CA_DEFAULT_HOSTNAME <settings-ca-default-hostname>` with a
+            Override the URL configured with :ref:`CA_DEFAULT_HOSTNAME <settings-ca-default-hostname>` with a
             different hostname. Set to ``False`` to disable the hostname.
-        pathlen : int, optional
+        path_length : int, optional
             Value of the path length attribute for the Basic Constraints extension.
         issuer_url : str
             URL for the DER/ASN1 formatted certificate that is signing certificates.
@@ -276,35 +313,21 @@ class CertificateAuthorityManager(
         ocsp_url : str, optional
             OCSP URL used for certificates signed by this CA. The default is no value, unless
             :ref:`CA_DEFAULT_HOSTNAME <settings-ca-default-hostname>` is set.
-        ca_issuer_url : str, optional
-            URL for the DER/ASN1 formatted certificate that is signing this CA. For intermediate CAs, this
-            would usually be the ``issuer_url`` of the parent CA.
-        ca_crl_url : list of str, optional
-            CRL URLs used for this CA. This value is only meaningful for intermediate CAs.
-        ca_ocsp_url : str, optional
-            OCSP URL used for this CA. This value is only meaningful for intermediate CAs. The default is
-            no value, unless :ref:`CA_DEFAULT_HOSTNAME <settings-ca-default-hostname>` is set.
-        permitted_subtrees : list of x509.GeneralName, optional
-            List of general names to add to the permitted names of the NameConstraints extension.
-        excluded_subtrees : list of x509.GeneralName, optional
-            List of general names to add to the permitted names of the NameConstraints extension.
-        name_constraints ``django_ca.extensions.NameConstraints``
-            Deprecated in favor of `permitted_subtrees` and `excluded_subtrees`.
         password : bytes or str, optional
             Password to encrypt the private key with.
         parent_password : bytes or str, optional
             Password that the private key of the parent CA is encrypted with.
-        ecc_curve : :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve`, optional
-            An elliptic curve to use for ECC keys. This parameter is ignored if ``key_type`` is not ``"ECC"``.
-            Defaults to the :ref:`CA_DEFAULT_ECC_CURVE <settings-ca-default-ecc-curve>`.
+        elliptic_curve : :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve`, optional
+            An elliptic curve to use for EC keys. This parameter is ignored if ``key_type`` is not ``"EC"``.
+            Defaults to the :ref:`CA_DEFAULT_ELLIPTIC_CURVE <settings-ca-default-elliptic-curve>`.
         key_type: str, optional
-            The type of private key to generate, must be one of ``"RSA"``, ``"DSA"``, ``"ECC"``, or
-            ``"EdDSA"`` , with ``"RSA"`` being the default.
+            The type of private key to generate, must be one of ``"RSA"``, ``"DSA"``, ``"EC"``, or
+            ``"Ed25519"`` , with ``"RSA"`` being the default.
         key_size : int, optional
             Integer specifying the key size, must be a power of two (e.g. 2048, 4096, ...). Defaults to
             the :ref:`CA_DEFAULT_KEY_SIZE <settings-ca-default-key-size>`, unused if
-            ``key_type="ECC"`` or ``key_type="EdDSA"``.
-        extra_extensions : list of :py:class:`cg:cryptography.x509.Extension`
+            ``key_type="EC"`` or ``key_type="Ed25519"``.
+        extensions : list of :py:class:`cg:cryptography.x509.Extension`
             An optional list of additional extensions to add to the certificate.
         path : str or pathlib.PurePath, optional
             Where to store the CA private key (default ``ca``).
@@ -316,6 +339,8 @@ class CertificateAuthorityManager(
             URL to the terms of service for the CA.
         acme_enabled : bool, optional
             Set to ``True`` to enable ACME support for this CA.
+        acme_profile : str, optional
+            The profile to use when issuing certificates via ACMEv2. Defaults to the CA_DEFAULT_PROFILE.
         acme_requires_contact : bool, optional
             Set to ``False`` if you do not want to force clients to register with an email address.
         openssh_ca : bool, optional
@@ -332,78 +357,109 @@ class CertificateAuthorityManager(
         # pylint: disable=too-many-arguments,too-many-locals,too-many-branches,too-many-statements
         # NOTE: Already verified by KeySizeAction, so these checks are only for when the Python API is used
         #       directly. generate_private_key() invokes this again, but we here to avoid sending a signal.
-        validate_key_parameters(key_size, key_type, ecc_curve)
 
-        if openssh_ca:
-            algorithm = None
-        elif algorithm is None:
-            algorithm = ca_settings.CA_DIGEST_ALGORITHM
+        if key_type == "ECC":  # type: ignore[comparison-overlap]  # that's a deprecated value
+            warnings.warn(
+                'key_type="ECC" is deprecated, use key_type="EC" instead.',
+                RemovedInDjangoCA126Warning,
+                stacklevel=2,
+            )
+            key_type = "EC"
+        if key_type == "EdDSA":  # type: ignore[comparison-overlap]  # that's a deprecated value
+            warnings.warn(
+                'key_type="EdDSA" key_type is deprecated, use key_type="Ed25519" instead.',
+                RemovedInDjangoCA126Warning,
+                stacklevel=2,
+            )
+            key_type = "Ed25519"
+
+        if isinstance(ca_ocsp_url, str):  # pragma: django_ca<1.26.0
+            ca_ocsp_url = [ca_ocsp_url]
+        if isinstance(ca_issuer_url, str):  # pragma: django_ca<1.26.0
+            ca_issuer_url = [ca_issuer_url]
+        if extensions is None:
+            extensions = []
+        else:
+            extensions = list(extensions)  # cast extensions to list if set (so that we can extend later)
+
+            # check type of values to provide better errors
+            for extension in extensions:
+                if isinstance(extension, x509.Extension) is False:
+                    raise ValueError(f"Cannot add extension of type {type(extension).__name__}")
+
+        key_size, elliptic_curve = validate_private_key_parameters(key_type, key_size, elliptic_curve)
+        algorithm = validate_public_key_parameters(key_type, algorithm)
+
         expires = parse_expires(expires)
 
         if openssh_ca and parent:
             raise ValueError("OpenSSH does not support intermediate authorities")
-        if not openssh_ca and key_type == "EdDSA":
-            raise ValueError("EdDSA only supported for OpenSSH authorities")
-
-        # Cast extra_extensions to list if set (so that we can extend if necessary)
-        if extra_extensions:
-            extra_extensions = list(extra_extensions)
-        else:
-            extra_extensions = []
 
         # Append OpenSSH extensions if an OpenSSH CA was requested
         if openssh_ca:
-            extra_extensions.extend([SshHostCaExtension(), SshUserCaExtension()])
+            extensions.extend([SshHostCaExtension(), SshUserCaExtension()])
 
-        # Normalize extensions to django_ca.extensions.Extension subclasses
-        if isinstance(issuer_alt_name, str):
-            issuer_alt_name = x509.Extension(
-                oid=x509.IssuerAlternativeName.oid,
-                critical=EXTENSION_DEFAULT_CRITICAL[x509.IssuerAlternativeName.oid],
-                value=x509.IssuerAlternativeName(general_names=[parse_general_name(issuer_alt_name)]),
+        extensions_dict = {ext.oid: ext for ext in extensions}
+
+        if ExtensionOID.KEY_USAGE not in extensions_dict:
+            extensions_dict[ExtensionOID.KEY_USAGE] = x509.Extension(
+                oid=ExtensionOID.KEY_USAGE,
+                critical=constants.EXTENSION_DEFAULT_CRITICAL[ExtensionOID.KEY_USAGE],
+                value=self.model.DEFAULT_KEY_USAGE,
             )
-        elif isinstance(issuer_alt_name, IssuerAlternativeName):
-            issuer_alt_name = issuer_alt_name.as_extension()
 
         if crl_url is None:
             crl_url = []
 
         serial = x509.random_serial_number()
-        hex_serial = int_to_hex(serial)
 
         if default_hostname is None:
             default_hostname = ca_settings.CA_DEFAULT_HOSTNAME
+        if acme_profile is None:
+            acme_profile = ca_settings.CA_DEFAULT_PROFILE
+        elif acme_profile not in ca_settings.CA_PROFILES:
+            raise ValueError(f"{acme_profile}: Profile is not defined.")
+
+        if parent:
+            signer_serial = parent.pub.loaded.serial_number
+        else:
+            signer_serial = serial
+
+        context = self._get_formatting_context(serial, signer_serial)
 
         # If there is a default hostname, use it to compute some URLs from that
         if isinstance(default_hostname, str) and default_hostname != "":
             default_hostname = validate_hostname(default_hostname, allow_port=True)
-            if parent:
-                root_serial = parent.serial
-            else:
-                root_serial = hex_serial
 
             # Set OCSP urls
             if not ocsp_url:
-                ocsp_path = reverse("django_ca:ocsp-cert-post", kwargs={"serial": hex_serial})
+                ocsp_path = reverse("django_ca:ocsp-cert-post", kwargs={"serial": context["SERIAL_HEX"]})
                 ocsp_url = f"http://{default_hostname}{ocsp_path}"
             if parent and not ca_ocsp_url:  # OCSP for CA only makes sense in intermediate CAs
-                ocsp_path = reverse("django_ca:ocsp-ca-post", kwargs={"serial": root_serial})
-                ca_ocsp_url = f"http://{default_hostname}{ocsp_path}"
+                ca_ocsp_url = [f"http://{default_hostname}/{context['OCSP_PATH']}"]
 
             # Set issuer path
-            issuer_path = reverse("django_ca:issuer", kwargs={"serial": root_serial})
             if parent and not ca_issuer_url:
-                ca_issuer_url = f"http://{default_hostname}{issuer_path}"
+                ca_issuer_url = [f"http://{default_hostname}/{context['CA_ISSUER_PATH']}"]
             if not issuer_url:
-                issuer_url = f"http://{default_hostname}{issuer_path}"
+                issuer_url = f"http://{default_hostname}/{context['CA_ISSUER_PATH']}"
 
             # Set CRL URLs
             if not crl_url:
-                crl_path = reverse("django_ca:crl", kwargs={"serial": hex_serial})
+                crl_path = reverse("django_ca:crl", kwargs={"serial": context["SERIAL_HEX"]})
                 crl_url = [f"http://{default_hostname}{crl_path}"]
             if parent and not ca_crl_url:  # CRL for CA only makes sense in intermediate CAs
-                ca_crl_path = reverse("django_ca:ca-crl", kwargs={"serial": root_serial})
-                ca_crl_url = [f"http://{default_hostname}{ca_crl_path}"]
+                ca_crl_url = [f"http://{default_hostname}/{context['CRL_PATH']}"]
+
+        # Handle the Authority Information Access extension by adding any manually passed access descriptions
+        self._handle_authority_information_access(extensions_dict, ca_issuer_url, ca_ocsp_url)
+
+        # Format extension values
+        format_extensions(extensions_dict, context)
+
+        # Cast extensions_dict back to list, so that signal handler receives the same type as the method
+        # itself. This has the added bonus of signal handlers being able to influence the extension order.
+        extensions = list(extensions_dict.values())
 
         pre_create_ca.send(
             sender=self.model,
@@ -414,7 +470,7 @@ class CertificateAuthorityManager(
             expires=expires,
             parent=parent,
             subject=subject,
-            pathlen=pathlen,
+            pathlen=path_length,
             issuer_url=issuer_url,
             issuer_alt_name=issuer_alt_name,
             crl_url=crl_url,
@@ -422,39 +478,27 @@ class CertificateAuthorityManager(
             ca_issuer_url=ca_issuer_url,
             ca_crl_url=ca_crl_url,
             ca_ocsp_url=ca_ocsp_url,
-            name_constraints=name_constraints,
             permitted_subtrees=permitted_subtrees,
             excluded_subtrees=excluded_subtrees,
             password=password,
             parent_password=parent_password,
-            extra_extensions=extra_extensions,
+            extensions=extensions,
             caa=caa,
             website=website,
             terms_of_service=terms_of_service,
             acme_enabled=acme_enabled,
+            acme_profile=acme_profile,
             acme_requires_contact=acme_requires_contact,
         )
 
-        private_key = generate_private_key(key_size, key_type, ecc_curve)
+        private_key = generate_private_key(key_size, key_type, elliptic_curve)
         public_key = private_key.public_key()
 
         builder = get_cert_builder(expires, serial=serial)
         builder = builder.public_key(public_key)
         builder = builder.subject_name(subject)
-        builder = builder.add_extension(x509.BasicConstraints(ca=True, path_length=pathlen), critical=True)
         builder = builder.add_extension(
-            x509.KeyUsage(
-                key_cert_sign=True,
-                crl_sign=True,
-                digital_signature=False,
-                content_commitment=False,
-                key_encipherment=False,
-                data_encipherment=False,
-                key_agreement=False,
-                encipher_only=False,
-                decipher_only=False,
-            ),
-            critical=True,
+            x509.BasicConstraints(ca=True, path_length=path_length), critical=True
         )
 
         subject_key_id = x509.SubjectKeyIdentifier.from_public_key(public_key)
@@ -470,21 +514,18 @@ class CertificateAuthorityManager(
             aki = parent.get_authority_key_identifier()
         builder = builder.add_extension(aki, critical=False)
 
-        for critical, ext in self.get_common_extensions(ca_issuer_url, ca_crl_url, ca_ocsp_url):
-            builder = builder.add_extension(ext, critical=critical)
+        for critical, ext in self.get_common_extensions(ca_crl_url):
+            # Check if the extension was passed directly, in which case we do not add it here.
+            if ext.oid not in extensions_dict:
+                builder = builder.add_extension(ext, critical=critical)
 
         if permitted_subtrees is not None or excluded_subtrees is not None:
             builder = builder.add_extension(
                 x509.NameConstraints(permitted_subtrees, excluded_subtrees), critical=True
             )
-        elif name_constraints:
-            if not isinstance(name_constraints, NameConstraints):
-                name_constraints = NameConstraints(name_constraints)
 
-            builder = builder.add_extension(*name_constraints.for_builder())
-
-        if extra_extensions:
-            builder = self._extra_extensions(builder, extra_extensions)
+        for extra_extension in extensions:
+            builder = builder.add_extension(extra_extension.value, critical=extra_extension.critical)
 
         certificate = builder.sign(private_key=private_sign_key, algorithm=algorithm)
 
@@ -507,6 +548,7 @@ class CertificateAuthorityManager(
             website=website,
             terms_of_service=terms_of_service,
             acme_enabled=acme_enabled,
+            acme_profile=acme_profile,
             acme_requires_contact=acme_requires_contact,
         )
         ca.update_certificate(certificate)
@@ -537,7 +579,7 @@ class CertificateManager(
 ):
     """Model manager for the Certificate model."""
 
-    if TYPE_CHECKING:
+    if typing.TYPE_CHECKING:
         # See CertificateManagerMixin for description on this branch
         #
         # pylint: disable=missing-function-docstring; just defining stubs here
@@ -604,7 +646,7 @@ class CertificateManager(
 class AcmeAccountManager(AcmeAccountManagerBase):
     """Model manager for :py:class:`~django_ca.models.AcmeAccount`."""
 
-    if TYPE_CHECKING:
+    if typing.TYPE_CHECKING:
         # See CertificateManagerMixin for description on this branch
         #
         # pylint: disable=missing-function-docstring; just defining stubs here
