@@ -1,15 +1,15 @@
 # This file is part of django-ca (https://github.com/mathiasertl/django-ca).
 #
-# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
-# General Public License as published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 #
-# You should have received a copy of the GNU General Public License along with django-ca.  If not,
-# see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with django-ca. If not, see
+# <http://www.gnu.org/licenses/>.
 
 """TestCase base classes that preload some data and add common helper methods."""
 
@@ -22,26 +22,28 @@ import tempfile
 import typing
 from contextlib import contextmanager
 from datetime import datetime, timedelta
+from datetime import timezone as tz
+from typing import Any, Dict, Iterable, Iterator, Tuple
 from unittest.mock import patch
 
 import cryptography
 from cryptography import x509
 from cryptography.hazmat.primitives import serialization
-from cryptography.hazmat.primitives.serialization import Encoding
 
 from django.conf import settings
 from django.test.utils import override_settings
+from django.utils import timezone
 
-from ...extensions import KEY_TO_EXTENSION, parse_extension
-from ...profiles import profiles
-from ...typehints import PrivateKeyTypes, TypedDict
-from ...utils import add_colons, ca_storage
-
-FuncTypeVar = typing.TypeVar("FuncTypeVar", bound=typing.Callable[..., typing.Any])
-KeyDict = TypedDict("KeyDict", {"pem": str, "parsed": PrivateKeyTypes})
-CsrDict = TypedDict("CsrDict", {"pem": str, "parsed": x509.CertificateSigningRequest, "der": bytes})
-_PubDict = TypedDict("_PubDict", {"pem": str, "parsed": x509.Certificate})
+from django_ca import constants
+from django_ca.extensions import parse_extension
+from django_ca.profiles import profiles
+from django_ca.typehints import CertificateIssuerPrivateKeyTypes
+from django_ca.utils import add_colons, ca_storage, format_name
 
+FuncTypeVar = typing.TypeVar("FuncTypeVar", bound=typing.Callable[..., Any])
+KeyDict = typing.TypedDict("KeyDict", {"pem": str, "parsed": CertificateIssuerPrivateKeyTypes})
+CsrDict = typing.TypedDict("CsrDict", {"pem": str, "parsed": x509.CertificateSigningRequest})
+_PubDict = typing.TypedDict("_PubDict", {"pem": str, "parsed": x509.Certificate})
 
 # Regex used by certbot to split PEM-encodied certificate chains/bundles as of 2022-01-23. See also:
 # 	https://github.com/certbot/certbot/blob/master/certbot/certbot/crypto_util.py
@@ -53,59 +55,60 @@ CERT_PEM_REGEX = re.compile(
     re.DOTALL,  # DOTALL (/s) because the base64text may include newlines
 )
 
+# If the installed cryptography version supports the unsafe_skip_rsa_key_validation option when loading
+# private keys
+SKIP_KEY_VALIDATION = settings.CRYPTOGRAPHY_VERSION >= (39, 0)
 
+
+# pylint: disable-next=inherit-non-class; False positive
 class PubDict(_PubDict, total=False):  # pylint: disable=missing-class-docstring
     der: bytes
 
 
-def _load_key(data: typing.Dict[typing.Any, typing.Any]) -> KeyDict:
-    basedir = data.get("basedir", settings.FIXTURES_DIR)
-    path = os.path.join(basedir, data["key_filename"])
-
-    with open(path, "rb") as stream:
+def _load_key(data: Dict[Any, Any]) -> KeyDict:
+    with open(os.path.join(settings.FIXTURES_DIR, data["key_filename"]), "rb") as stream:
         raw = stream.read()
 
-    parsed = serialization.load_pem_private_key(raw, password=data.get("password"))
+    if SKIP_KEY_VALIDATION is True:  # pragma: only cryptography>=39.0
+        parsed = serialization.load_pem_private_key(
+            raw, password=data.get("password"), unsafe_skip_rsa_key_validation=True
+        )
+    else:  # pragma: only cryptography<39.0
+        parsed = serialization.load_pem_private_key(raw, password=data.get("password"))
     return {
         "pem": raw.decode("utf-8"),
         "parsed": parsed,  # type: ignore[typeddict-item]  # we do not support all key types
     }
 
 
-def _load_csr(data: typing.Dict[typing.Any, typing.Any]) -> CsrDict:
-    basedir = data.get("basedir", settings.FIXTURES_DIR)
-    path = os.path.join(basedir, data["csr_filename"])
-
-    with open(path, "rb") as stream:
-        raw = stream.read().strip()
+def _load_csr(data: Dict[Any, Any]) -> CsrDict:
+    with open(os.path.join(settings.FIXTURES_DIR, data["csr_filename"]), "rb") as stream:
+        raw = stream.read()
 
     parsed = x509.load_pem_x509_csr(raw)
     return {
         "pem": raw.decode("utf-8"),
         "parsed": parsed,
-        "der": parsed.public_bytes(Encoding.DER),
     }
 
 
-def _load_pub(data: typing.Dict[typing.Any, typing.Any]) -> PubDict:
+def _load_pub(data: Dict[Any, Any]) -> PubDict:
+    # basedir is set for certificates in docs/source/_files
     basedir = data.get("basedir", settings.FIXTURES_DIR)
     path = os.path.join(basedir, data["pub_filename"])
 
     with open(path, "rb") as stream:
-        pem = stream.read().replace(b"\r\n", b"\n")
+        pem = stream.read()
 
     pub_data: PubDict = {
         "pem": pem.decode("utf-8"),
         "parsed": x509.load_pem_x509_certificate(pem),
     }
 
-    if data.get("pub_der_filename"):
-        der_path = os.path.join(basedir, data["pub_der_filename"])
-        with open(der_path, "rb") as stream:
-            der = stream.read().replace(b"\r\n", b"\n")
+    if der_filename := data.get("pub_der_filename"):
+        with open(os.path.join(basedir, der_filename), "rb") as stream:
+            der = stream.read()
         pub_data["der"] = der
-        # Fails for alt-extensions since alternative AKI was added
-        # pub_data['der_parsed'] = x509.load_der_x509_certificate(der),
 
     return pub_data
 
@@ -119,6 +122,14 @@ certs = _fixture_data.get("certs")
 # Update some data from contrib (data is not in cert-data.json, since we don't generate them)
 certs["multiple_ous"] = {
     "name": "multiple_ous",
+    "subject": [
+        ["C", "US"],
+        ["O", "VeriSign, Inc."],
+        ["OU", "Class 3 Public Primary Certification Authority - G2"],
+        ["OU", "(c) 1998 VeriSign, Inc. - For authorized use only"],
+        ["OU", "VeriSign Trust Network"],
+    ],
+    "subject_str": "/C=US/O=VeriSign, Inc./OU=Class 3 Public Primary Certification Authority - G2/OU=(c) 1998 VeriSign, Inc. - For authorized use only/OU=VeriSign Trust Network",  # noqa: E501
     "cn": "",
     "key_filename": False,
     "csr_filename": False,
@@ -133,18 +144,24 @@ certs["multiple_ous"] = {
     "hpkp": "AjyBzOjnxk+pQtPBUEhwfTXZu1uH9PVExb8bxWQ68vo=",
     "md5": "A2:33:9B:4C:74:78:73:D4:6C:E7:C1:F3:8D:CB:5C:E9",
     "sha1": "85:37:1C:A6:E5:50:14:3D:CE:28:03:47:1B:DE:3A:09:E8:F8:77:0F",
-    "sha256": "83:CE:3C:12:29:68:8A:59:3D:48:5F:81:97:3C:0F:91:95:43:1E:DA:37:CC:5E:36:43:0E:79:C7:A8:88:63:8B",  # NOQA
+    "sha256": "83:CE:3C:12:29:68:8A:59:3D:48:5F:81:97:3C:0F:91:95:43:1E:DA:37:CC:5E:36:43:0E:79:C7:A8:88:63:8B",  # noqa: E501
     "sha512": "86:20:07:9F:8B:06:80:43:44:98:F6:7A:A4:22:DE:7E:2B:33:10:9B:65:72:79:C4:EB:F3:F3:0F:66:C8:6E:89:1D:4C:6C:09:1C:83:45:D1:25:6C:F8:65:EB:9A:B9:50:8F:26:A8:85:AE:3A:E4:8A:58:60:48:65:BB:44:B6:CE",  # NOQA
 }
 certs["cloudflare_1"] = {
     "name": "cloudflare_1",
+    "subject": [
+        ["OU", "Domain Control Validated"],
+        ["OU", "PositiveSSL Multi-Domain"],
+        ["CN", "sni24142.cloudflaressl.com"],
+    ],
+    "subject_str": "/OU=Domain Control Validated/OU=PositiveSSL Multi-Domain/CN=sni24142.cloudflaressl.com",
     "cn": "sni24142.cloudflaressl.com",
     "key_filename": False,
     "csr_filename": False,
     "pub_filename": os.path.join("contrib", "cloudflare_1.pem"),
     "cat": "contrib",
     "type": "cert",
-    "key_type": "ECC",
+    "key_type": "EC",
     "valid_from": "2018-07-18 00:00:00",
     "valid_until": "2019-01-24 23:59:59",
     "ca": "root",
@@ -185,7 +202,7 @@ certs["cloudflare_1"] = {
     },
     "key_usage": {
         "critical": True,
-        "value": ["digitalSignature"],
+        "value": ["digital_signature"],
     },
     "precert_poison": {"critical": True},
     "subject_alternative_name": {
@@ -303,6 +320,7 @@ for cert_name, cert_data in certs.items():
 
     if cert_data["type"] == "ca":
         cert_data.setdefault("children", [])
+        cert_data["children"] = [(k, add_colons(v)) for k, v in cert_data["children"]]
 
     # Load data from files
     if cert_data["key_filename"] is not False:
@@ -310,18 +328,26 @@ for cert_name, cert_data in certs.items():
     if cert_data["csr_filename"] is not False:
         cert_data["csr"] = _load_csr(cert_data)
     cert_data["pub"] = _load_pub(cert_data)
+    cert_data["issuer"] = cert_data["pub"]["parsed"].issuer
+    cert_data["issuer_str"] = format_name(cert_data["issuer"])
+
+    if "subject" not in cert_data:  # pragma: no cover
+        raise ValueError(f"subject not in {cert_name}")
+    if "subject_str" not in cert_data:  # pragma: no cover
+        raise ValueError(f"subject_str not in {cert_name}")
+    if not isinstance(cert_data["subject"], list):  # pragma: no cover
+        raise ValueError(cert_data["subject"])
+    if not isinstance(cert_data["subject_str"], str):  # pragma: no cover
+        raise ValueError(cert_data["subject_str"])
 
     # parse some data from the dict
     cert_data["valid_from"] = datetime.strptime(cert_data["valid_from"], "%Y-%m-%d %H:%M:%S")
     cert_data["valid_until"] = datetime.strptime(cert_data["valid_until"], "%Y-%m-%d %H:%M:%S")
-    cert_data["valid_from_short"] = cert_data["valid_from"].strftime("%Y-%m-%d %H:%M")
-    cert_data["valid_until_short"] = cert_data["valid_until"].strftime("%Y-%m-%d %H:%M")
-
-    cert_data["ocsp-serial"] = cert_data["serial"].replace(":", "")
-    cert_data["ocsp-expires"] = cert_data["valid_until"].strftime("%y%m%d%H%M%SZ")
+    cert_data["valid_from_str"] = cert_data["valid_from"].replace(tzinfo=tz.utc).isoformat(" ")
+    cert_data["valid_until_str"] = cert_data["valid_until"].replace(tzinfo=tz.utc).isoformat(" ")
 
     # parse extensions
-    for ext_key, ext_cls in KEY_TO_EXTENSION.items():
+    for ext_key in constants.EXTENSION_KEY_OIDS:
         if cert_data.get(ext_key):
             cert_data[f"{ext_key}_serialized"] = cert_data[ext_key]
 
@@ -334,15 +360,18 @@ for cert_name, cert_data in certs.items():
 
 # Calculate some fixed timestamps that we reuse throughout the tests
 timestamps = {
-    "base": datetime.strptime(_fixture_data["timestamp"], "%Y-%m-%d %H:%M:%S"),
+    "base": datetime.fromisoformat(_fixture_data["timestamp"]),
+    "before_everything": datetime(1990, 1, 1, tzinfo=tz.utc),
 }
-timestamps["before_everything"] = datetime(1990, 1, 1)
 timestamps["before_cas"] = timestamps["base"] - timedelta(days=1)
 timestamps["before_child"] = timestamps["base"] + timedelta(days=1)
 timestamps["after_child"] = timestamps["base"] + timedelta(days=4)
 timestamps["ca_certs_valid"] = timestamps["base"] + timedelta(days=7)
 timestamps["profile_certs_valid"] = timestamps["base"] + timedelta(days=12)
-timestamps["everything_valid"] = timestamps["base"] + timedelta(days=60)
+
+# When creating fixutres, latest valid_from of any generated cert is 20 days, we need to be after that
+timestamps["everything_valid"] = timestamps["base"] + timedelta(days=23)
+timestamps["everything_valid_naive"] = timezone.make_naive(timestamps["everything_valid"])
 timestamps["cas_expired"] = timestamps["base"] + timedelta(days=731, seconds=3600)
 timestamps["ca_certs_expiring"] = certs["root-cert"]["valid_until"] - timedelta(days=3)
 timestamps["ca_certs_expired"] = certs["root-cert"]["valid_until"] + timedelta(seconds=3600)
@@ -362,14 +391,14 @@ def uri(url: str) -> x509.UniformResourceIdentifier:  # just a shortcut
 
 
 def rdn(
-    name: typing.Iterable[typing.Tuple[x509.ObjectIdentifier, str]]
+    name: Iterable[Tuple[x509.ObjectIdentifier, str]]
 ) -> x509.RelativeDistinguishedName:  # just a shortcut
     """Shortcut to get a :py:class:`cg:cryptography.x509.RelativeDistinguishedName`."""
     return x509.RelativeDistinguishedName([x509.NameAttribute(*t) for t in name])
 
 
 @contextmanager
-def mock_cadir(path: str) -> typing.Iterator[None]:
+def mock_cadir(path: str) -> Iterator[None]:
     """Contextmanager to set the CA_DIR to a given path without actually creating it."""
     with override_settings(CA_DIR=path), patch.object(ca_storage, "location", path), patch.object(
         ca_storage, "_location", path
@@ -386,7 +415,7 @@ class override_tmpcadir(override_settings):  # pylint: disable=invalid-name; in
     def __call__(self, test_func: FuncTypeVar) -> FuncTypeVar:
         if not inspect.isfunction(test_func):
             raise ValueError("Only functions can use override_tmpcadir()")
-        return super().__call__(test_func)  # type: ignore[no-any-return]
+        return super().__call__(test_func)  # type: ignore[return-value]  # cannot figure out whats right here
 
     def enable(self) -> None:
         self.options["CA_DIR"] = tempfile.mkdtemp()
@@ -418,6 +447,3 @@ class override_tmpcadir(override_settings):  # pylint: disable=invalid-name; in
         self.mock.stop()
         self.mock_.stop()
         shutil.rmtree(self.options["CA_DIR"])
-
-
-__all__ = ("override_settings",)
