@@ -1,15 +1,15 @@
 # This file is part of django-ca (https://github.com/mathiasertl/django-ca).
 #
-# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
-# General Public License as published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 #
-# You should have received a copy of the GNU General Public License along with django-ca.  If not,
-# see <http://www.gnu.org/licenses/>
+# You should have received a copy of the GNU General Public License along with django-ca. If not, see
+# <http://www.gnu.org/licenses/>.
 
 """Collection of mixin classes for unittest.TestCase subclasses."""
 
@@ -19,11 +19,10 @@ import json
 import re
 import textwrap
 import typing
-import warnings
 from contextlib import contextmanager
 from datetime import datetime, timedelta
 from http import HTTPStatus
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
+from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, Type, Union
 from unittest import mock
 from urllib.parse import quote
 
@@ -45,7 +44,6 @@ from django.core.management import ManagementUtility, call_command
 from django.core.management.base import CommandError
 from django.db import models
 from django.dispatch.dispatcher import Signal
-from django.http import HttpResponse
 from django.templatetags.static import static
 from django.test.testcases import SimpleTestCase
 from django.urls import reverse
@@ -53,29 +51,30 @@ from django.urls import reverse
 from freezegun import freeze_time
 from freezegun.api import FrozenDateTimeFactory, StepTickTimeFactory
 
-from ... import ca_settings
-from ...constants import ReasonFlags
-from ...deprecation import RemovedInDjangoCA124Warning
-from ...extensions import extension_as_text
-from ...models import Certificate, CertificateAuthority, DjangoCAModel, X509CertMixin
-from ...signals import (
+from django_ca import ca_settings, constants
+from django_ca.constants import ReasonFlags
+from django_ca.deprecation import RemovedInDjangoCA126Warning, RemovedInDjangoCA127Warning
+from django_ca.extensions import extension_as_text
+from django_ca.models import Certificate, CertificateAuthority, DjangoCAModel, X509CertMixin
+from django_ca.signals import (
     post_create_ca,
     post_issue_cert,
     post_revoke_cert,
     post_sign_cert,
     pre_create_ca,
-    pre_issue_cert,
     pre_sign_cert,
 )
-from ...utils import ca_storage, parse_general_name
-from . import certs, timestamps, uri
-from .typehints import DjangoCAModelTypeVar
+from django_ca.tests.base import certs, timestamps, uri
+from django_ca.tests.base.typehints import DjangoCAModelTypeVar
+from django_ca.utils import add_colons, ca_storage, parse_general_name
 
 if typing.TYPE_CHECKING:
     # Use SimpleTestCase as base class when type checking. This way mypy will know about attributes/methods
     # that the mixin accesses. See also:
     #   https://github.com/python/mypy/issues/5837
     TestCaseProtocol = SimpleTestCase
+
+    from django.test.client import _MonkeyPatchedWSGIResponse as HttpResponse
 else:
     TestCaseProtocol = object
 
@@ -95,6 +94,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
 
     def setUp(self) -> None:
         # Add custom equality functions
+        self.addTypeEqualityFunc(x509.AuthorityInformationAccess, self.assertAuthorityInformationAccessEqual)
         self.addTypeEqualityFunc(x509.ExtendedKeyUsage, self.assertExtendedKeyUsageEqual)
         self.addTypeEqualityFunc(x509.Extension, self.assertCryptographyExtensionEqual)
         self.addTypeEqualityFunc(x509.KeyUsage, self.assertKeyUsageEqual)
@@ -110,7 +110,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
 
         # Set `self.ca` as a default certificate authority (if at least one is loaded)
         if len(self.load_cas) == 1:  # only one CA specified, set self.ca for convenience
-            self.ca = self.cas[self.load_cas[0]]
+            self.ca: CertificateAuthority = self.cas[self.load_cas[0]]
         elif self.load_cas:
             if self.default_ca not in self.load_cas:  # pragma: no cover
                 self.fail(f"{self.default_ca}: Not in {self.load_cas}.")
@@ -188,7 +188,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         self.assertEqual(actual.key_identifier, expected.key_identifier)
 
     def assertBasic(  # pylint: disable=invalid-name
-        self, cert: x509.Certificate, algo: typing.Type[hashes.HashAlgorithm] = hashes.SHA256
+        self, cert: x509.Certificate, algo: Type[hashes.HashAlgorithm] = hashes.SHA256
     ) -> None:
         """Assert some basic key properties."""
         self.assertEqual(cert.version, x509.Version.v3)
@@ -216,10 +216,16 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             The raw CRL
         expected : list
             List of CAs/certs to be expected in this CRL
+        signer
+        expires
+        algorithm
+        encoding
+        idp
+        extensions
+        crl_number
         """
         expected = expected or []
         signer = signer or self.cas["child"]
-        algorithm = algorithm or ca_settings.CA_DIGEST_ALGORITHM
         extensions = extensions or []
         expires_timestamp = datetime.utcnow() + timedelta(seconds=expires)
 
@@ -242,7 +248,13 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             raise TypeError()  # just to make mypy happy
 
         self.assertIsInstance(parsed_crl.signature_hash_algorithm, type(algorithm))
-        self.assertTrue(parsed_crl.is_signature_valid(public_key))
+
+        # cryptography can not validate signatures of CRLs for non DSA/RSA/EC keys
+        # https://github.com/pyca/cryptography/issues/8156
+        if settings.CRYPTOGRAPHY_VERSION >= (40, 0) or not isinstance(  # pragma: cryptography<40.0 branch
+            public_key, (ed448.Ed448PublicKey, ed25519.Ed25519PublicKey)
+        ):
+            self.assertTrue(parsed_crl.is_signature_valid(public_key))
         self.assertEqual(parsed_crl.issuer, signer.pub.loaded.subject)
         self.assertEqual(parsed_crl.last_update, datetime.utcnow())
         self.assertEqual(parsed_crl.next_update, expires_timestamp)
@@ -255,7 +267,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             self.assertEqual(list(entry.extensions), [])
 
     @contextmanager
-    def assertCommandError(self, msg: str) -> typing.Iterator[None]:  # pylint: disable=invalid-name
+    def assertCommandError(self, msg: str) -> Iterator[None]:  # pylint: disable=invalid-name
         """Context manager asserting that CommandError is raised.
 
         Parameters
@@ -269,11 +281,11 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
     @contextmanager
     def assertCreateCASignals(  # pylint: disable=invalid-name
         self, pre: bool = True, post: bool = True
-    ) -> typing.Iterator[Tuple[mock.Mock, mock.Mock]]:
+    ) -> Iterator[Tuple[mock.Mock, mock.Mock]]:
         """Context manager mocking both pre and post_create_ca signals."""
         with self.mockSignal(pre_create_ca) as pre_sig, self.mockSignal(post_create_ca) as post_sig:
             try:
-                yield (pre_sig, post_sig)
+                yield pre_sig, post_sig
             finally:
                 self.assertTrue(pre_sig.called is pre)
                 self.assertTrue(post_sig.called is post)
@@ -281,15 +293,24 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
     @contextmanager
     def assertSignCertSignals(  # pylint: disable=invalid-name
         self, pre: bool = True, post: bool = True
-    ) -> typing.Iterator[Tuple[mock.Mock, mock.Mock]]:
+    ) -> Iterator[Tuple[mock.Mock, mock.Mock]]:
         """Context manager mocking both pre and post_create_ca signals."""
         with self.mockSignal(pre_sign_cert) as pre_sig, self.mockSignal(post_sign_cert) as post_sig:
             try:
-                yield (pre_sig, post_sig)
+                yield pre_sig, post_sig
             finally:
                 self.assertTrue(pre_sig.called is pre)
                 self.assertTrue(post_sig.called is post)
 
+    def assertAuthorityInformationAccessEqual(  # pylint: disable=invalid-name
+        self,
+        first: x509.AuthorityInformationAccess,
+        second: x509.AuthorityInformationAccess,
+        msg: Optional[str] = None,
+    ) -> None:
+        """Type equality function for x509.AuthorityInformationAccess."""
+        self.assertEqual(list(first), list(second), msg=msg)
+
     def assertCryptographyExtensionEqual(  # pylint: disable=invalid-name
         self,
         first: x509.Extension[x509.ExtensionType],
@@ -351,11 +372,11 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
     @contextmanager
     def assertCreateCertSignals(  # pylint: disable=invalid-name
         self, pre: bool = True, post: bool = True
-    ) -> typing.Iterator[Tuple[mock.Mock, mock.Mock]]:
+    ) -> Iterator[Tuple[mock.Mock, mock.Mock]]:
         """Context manager mocking both pre and post_create_ca signals."""
-        with self.mockSignal(pre_issue_cert) as pre_sig, self.mockSignal(post_issue_cert) as post_sig:
+        with self.mockSignal(pre_sign_cert) as pre_sig, self.mockSignal(post_issue_cert) as post_sig:
             try:
-                yield (pre_sig, post_sig)
+                yield pre_sig, post_sig
             finally:
                 self.assertTrue(pre_sig.called is pre)
                 self.assertTrue(post_sig.called is post)
@@ -433,13 +454,13 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
                 x509.Extension(oid=ExtensionOID.SUBJECT_KEY_IDENTIFIER, critical=False, value=ski),
             )
 
-        # Diff output is really bad for dicts, so we sort this based on dotted string to get better output
+        # Diff output is bad for dicts, so we sort this based on dotted string to get better output
         actual_tuple = sorted(actual.items(), key=lambda t: t[0].dotted_string)
         expected_tuple = sorted(expected.items(), key=lambda t: t[0].dotted_string)
         self.assertEqual(actual_tuple, expected_tuple)
 
     @contextmanager
-    def assertImproperlyConfigured(self, msg: str) -> typing.Iterator[None]:  # pylint: disable=invalid-name
+    def assertImproperlyConfigured(self, msg: str) -> Iterator[None]:  # pylint: disable=invalid-name
         """Shortcut for testing that the code raises ImproperlyConfigured with the given message."""
         with self.assertRaisesRegex(ImproperlyConfigured, msg):
             yield
@@ -451,7 +472,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         self.assertEqual(cert.issuer, issuer.subject)
 
     def assertMessages(  # pylint: disable=invalid-name
-        self, response: HttpResponse, expected: List[str]
+        self, response: "HttpResponse", expected: List[str]
     ) -> None:
         """Assert given Django messages for `response`."""
         messages = [str(m) for m in list(get_messages(response.wsgi_request))]
@@ -489,22 +510,19 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             self.assertTrue(key.key_size > 0)
 
     @contextmanager
-    def assertRemovedExtensionWarning(  # pylint: disable=invalid-name
-        self, name: str
-    ) -> typing.Iterator[None]:
-        """Temporary manager for removed extension wrapper classes."""
-        msg = rf"^django_ca\.extensions\.extensions\.{name} is deprecated and will be removed in django-ca 1\.24\.0\.$"  # NOQA
-        with self.assertWarnsRegex(RemovedInDjangoCA124Warning, msg):
+    def assertRemovedIn126Warning(self, msg: str) -> Iterator[None]:  # pylint: disable=invalid-name
+        """Assert that a RemovedInDjangoCA126Warning is thrown."""
+        with self.assertWarnsRegex(RemovedInDjangoCA126Warning, msg):
             yield
 
     @contextmanager
-    def assertRemovedIn124Warning(self, msg: str) -> typing.Iterator[None]:  # pylint: disable=invalid-name
-        """Assert that a RemovedInDjangoCA124Warning is thrown."""
-        with self.assertWarnsRegex(RemovedInDjangoCA124Warning, msg):
+    def assertRemovedIn127Warning(self, msg: str) -> Iterator[None]:  # pylint: disable=invalid-name
+        """Assert that a RemovedInDjangoCA127Warning is thrown."""
+        with self.assertWarnsRegex(RemovedInDjangoCA127Warning, msg):
             yield
 
     def assertRevoked(  # pylint: disable=invalid-name
-        self, cert: X509CertMixin, reason: Optional[str] = None
+        self, cert: X509CertMixin, reason: Optional[str] = None, compromised: Optional[datetime] = None
     ) -> None:
         """Assert that the certificate is now revoked."""
         if isinstance(cert, CertificateAuthority):
@@ -513,6 +531,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             cert = Certificate.objects.get(serial=cert.serial)
 
         self.assertTrue(cert.revoked)
+        self.assertEqual(cert.compromised, compromised)
 
         if reason is None:
             self.assertEqual(cert.revoked_reason, ReasonFlags.unspecified.name)
@@ -520,9 +539,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             self.assertEqual(cert.revoked_reason, reason)
 
     def assertSignature(  # pylint: disable=invalid-name
-        self,
-        chain: Iterable[CertificateAuthority],
-        cert: Union[Certificate, CertificateAuthority],
+        self, chain: Iterable[CertificateAuthority], cert: Union[Certificate, CertificateAuthority]
     ) -> None:
         """Assert that `cert` is properly signed by `chain`.
 
@@ -547,7 +564,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         self.assertIsNone(store_ctx.verify_certificate())  # type: ignore[func-returns-value]
 
     @contextmanager
-    def assertSystemExit(self, code: int) -> typing.Iterator[None]:  # pylint: disable=invalid-name
+    def assertSystemExit(self, code: int) -> Iterator[None]:  # pylint: disable=invalid-name
         """Assert that SystemExit is raised."""
         with self.assertRaisesRegex(SystemExit, rf"^{code}$") as excm:
             yield
@@ -556,7 +573,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
     @contextmanager
     def assertValidationError(  # pylint: disable=invalid-name; unittest standard
         self, errors: Dict[str, List[str]]
-    ) -> typing.Iterator[None]:
+    ) -> Iterator[None]:
         """Context manager to assert that a ValidationError is thrown."""
         with self.assertRaises(ValidationError) as cmex:
             yield
@@ -589,10 +606,18 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         )
 
     @property
-    def ca_certs(self) -> typing.Iterator[Tuple[str, Certificate]]:
+    def ca_certs(self) -> Iterator[Tuple[str, Certificate]]:
         """Yield loaded certificates for each certificate authority."""
         for name, cert in self.certs.items():
-            if name in ["root-cert", "child-cert", "ecc-cert", "dsa-cert", "pwd-cert"]:
+            if name in [
+                "root-cert",
+                "child-cert",
+                "ec-cert",
+                "dsa-cert",
+                "pwd-cert",
+                "ed448-cert",
+                "ed25519-cert",
+            ]:
                 yield name, cert
 
     def ca_issuers(self, issuer: x509.GeneralName) -> x509.AccessDescription:
@@ -729,7 +754,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             def _read_mock(size=None):  # type: ignore # pylint: disable=unused-argument
                 return stdin
 
-            # TYPE NOTE: mypy detects a different type, but important thing is its a context manager
+            # TYPE NOTE: mypy detects a different type, but important thing is it's a context manager
             stdin_mock = mock.patch(  # type: ignore[assignment]
                 "sys.stdin.buffer.read", side_effect=_read_mock
             )
@@ -775,6 +800,14 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         cert.full_clean()
         return cert
 
+    def certificate_policies(
+        self, *policies: x509.PolicyInformation, critical: bool = False
+    ) -> x509.Extension[x509.CertificatePolicies]:
+        """Shortcut for getting a Certificate Policy extension"""
+        return x509.Extension(
+            oid=ExtensionOID.CERTIFICATE_POLICIES, critical=critical, value=x509.CertificatePolicies(policies)
+        )
+
     def crl_distribution_points(
         self,
         full_name: Optional[Iterable[x509.GeneralName]] = None,
@@ -815,6 +848,14 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             oid=ExtensionOID.EXTENDED_KEY_USAGE, critical=critical, value=x509.ExtendedKeyUsage(usages)
         )
 
+    def ext(
+        self, value: x509.ExtensionType, critical: Optional[bool] = None
+    ) -> x509.Extension[x509.ExtensionType]:
+        """Shortcut to get a x509.Extension object from the given ExtensionType"""
+        if critical is None:  # pragma: no branch
+            critical = constants.EXTENSION_DEFAULT_CRITICAL[value.oid]
+        return x509.Extension(oid=value.oid, critical=critical, value=value)
+
     def freshest_crl(
         self,
         full_name: Optional[Iterable[x509.GeneralName]] = None,
@@ -896,7 +937,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         self,
         permitted: Optional[Iterable[x509.GeneralName]] = None,
         excluded: Optional[Iterable[x509.GeneralName]] = None,
-        critical: bool = False,
+        critical: bool = True,
     ) -> x509.Extension[x509.NameConstraints]:
         """Shortcut for getting a NameConstraints extension."""
         return x509.Extension(
@@ -923,12 +964,12 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         return x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, self.hostname)])
 
     def subject_alternative_name(
-        self, *names: x509.GeneralName
+        self, *names: x509.GeneralName, critical: bool = False
     ) -> x509.Extension[x509.SubjectAlternativeName]:
         """Shortcut for getting a SubjectAlternativeName extension."""
         return x509.Extension(
             oid=ExtensionOID.SUBJECT_ALTERNATIVE_NAME,
-            critical=False,
+            critical=critical,
             value=x509.SubjectAlternativeName(names),
         )
 
@@ -946,15 +987,14 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         )
 
     @classmethod
-    def expires(cls, days: int) -> datetime:
+    def expires(cls, days: int) -> timedelta:
         """Get a timestamp `days` from now."""
-        now = datetime.utcnow().replace(hour=0, minute=0, second=0, microsecond=0)
-        return now + timedelta(days + 1)
+        return timedelta(days=days + 1)
 
     @contextmanager
     def freeze_time(
         self, timestamp: Union[str, datetime]
-    ) -> typing.Iterator[Union[FrozenDateTimeFactory, StepTickTimeFactory]]:
+    ) -> Iterator[Union[FrozenDateTimeFactory, StepTickTimeFactory]]:
         """Context manager to freeze time to a given timestamp.
 
         If `timestamp` is a str that is in the `timestamps` dict (e.g. "everything-valid"), use that
@@ -969,17 +1009,17 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
     def get_cert_context(self, name: str) -> Dict[str, Any]:
         """Get a dictionary suitable for testing output based on the dictionary in basic.certs."""
         ctx: Dict[str, Any] = {}
-        for key, value in certs[name].items():
+        for key, value in sorted(certs[name].items()):
             # Handle cryptography extensions
             if key == "precert_poison":
-                ctx["precert_poison"] = "Precert Poison (critical):\n    Yes"
+                ctx["precert_poison"] = "* Precert Poison (critical):\n  Yes"
             elif isinstance(value, x509.Extension):
                 if value.critical:
                     ctx[f"{key}_critical"] = " (critical)"
                 else:
                     ctx[f"{key}_critical"] = ""
 
-                ctx[f"{key}_text"] = textwrap.indent(extension_as_text(value.value), "    ")
+                ctx[f"{key}_text"] = textwrap.indent(extension_as_text(value.value), "  ")
             elif key == "precertificate_signed_certificate_timestamps_serialized":
                 ctx["sct_critical"] = " (critical)" if value["critical"] else ""
                 ctx["sct_values"] = []
@@ -987,7 +1027,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
                     ctx["sct_values"].append(val)
             elif key == "precertificate_signed_certificate_timestamps":
                 continue  # special extension b/c it cannot be created
-            elif key == "pathlen":
+            elif key == "path_length":
                 ctx[key] = value
                 ctx[f"{key}_text"] = "unlimited" if value is None else value
             else:
@@ -997,6 +1037,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             parent = certs[certs[name]["parent"]]
             ctx["parent_name"] = parent["name"]
             ctx["parent_serial"] = parent["serial"]
+            ctx["parent_serial_colons"] = add_colons(parent["serial"])
 
         if certs[name]["key_filename"] is not False:
             ctx["key_path"] = ca_storage.path(certs[name]["key_filename"])
@@ -1044,8 +1085,9 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         return cert
 
     @contextmanager
-    def mockSignal(self, signal: Signal) -> typing.Iterator[mock.Mock]:  # pylint: disable=invalid-name
+    def mockSignal(self, signal: Signal) -> Iterator[mock.Mock]:  # pylint: disable=invalid-name
         """Context manager to attach a mock to the given signal."""
+
         # This function is only here to create an autospec. From the documentation:
         #
         #   Notice that the function takes a sender argument, along with wildcard keyword arguments
@@ -1064,7 +1106,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
             signal.disconnect(signal_mock)
 
     @contextmanager
-    def mute_celery(self, *calls: Any) -> typing.Iterator[mock.MagicMock]:
+    def mute_celery(self, *calls: Any) -> Iterator[mock.MagicMock]:
         """Context manager to mock celery invocations.
 
         This context manager mocks ``celery.app.task.Task.apply_async``, the final function in celery before
@@ -1097,40 +1139,33 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
         # Make sure that task was called the right number of times
         self.assertEqual(len(calls), len(mocked.call_args_list))
         for expected, actual in zip(calls, mocked.call_args_list):
-            self.assertEqual(expected, actual)
+            self.assertEqual(expected, actual, actual)
 
     @contextmanager
-    def patch(self, *args: Any, **kwargs: Any) -> typing.Iterator[mock.MagicMock]:
+    def patch(self, *args: Any, **kwargs: Any) -> Iterator[mock.MagicMock]:
         """Shortcut to :py:func:`py:unittest.mock.patch`."""
         with mock.patch(*args, **kwargs) as mocked:
             yield mocked
 
     @contextmanager
-    def patch_object(self, *args: Any, **kwargs: Any) -> typing.Iterator[Any]:
+    def patch_object(self, *args: Any, **kwargs: Any) -> Iterator[Any]:
         """Shortcut to :py:func:`py:unittest.mock.patch.object`."""
         with mock.patch.object(*args, **kwargs) as mocked:
             yield mocked
 
     def reverse(self, name: str, *args: Any, **kwargs: Any) -> str:
-        """Shortcut to reverse an URI name."""
+        """Shortcut to reverse a URI name."""
         return reverse(f"django_ca:{name}", args=args, kwargs=kwargs)
 
-    @contextmanager
-    def silence_warnings(self) -> typing.Iterator[None]:
-        """Contect manager to hide all warnings."""
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            yield
-
     @property
-    def usable_cas(self) -> typing.Iterator[Tuple[str, CertificateAuthority]]:
+    def usable_cas(self) -> Iterator[Tuple[str, CertificateAuthority]]:
         """Yield loaded generated certificates."""
         for name, ca in self.cas.items():
             if certs[name]["key_filename"]:
                 yield name, ca
 
     @property
-    def usable_certs(self) -> typing.Iterator[Tuple[str, Certificate]]:
+    def usable_certs(self) -> Iterator[Tuple[str, Certificate]]:
         """Yield loaded generated certificates."""
         for name, cert in self.certs.items():
             if certs[name]["cat"] == "generated":
@@ -1140,7 +1175,7 @@ class TestCaseMixin(TestCaseProtocol):  # pylint: disable=too-many-public-method
 class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
     """Common mixin for testing admin classes for models."""
 
-    model: typing.Type[DjangoCAModelTypeVar]
+    model: Type[DjangoCAModelTypeVar]
     """Model must be configured for TestCase instances using this mixin."""
 
     media_css: Tuple[str, ...] = tuple()
@@ -1150,18 +1185,18 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
     """The name of the view being tested."""
 
     # TODO: we should get rid of this, it's ugly
-    obj: Optional[DjangoCAModel]
+    obj: DjangoCAModelTypeVar
 
     def setUp(self) -> None:
         super().setUp()
         self.user = self.create_superuser()
         self.client.force_login(self.user)
-        self.obj = self.model.objects.first()  # TODO: get rid of this
+        self.obj = self.model.objects.first()  # type: ignore[assignment] # TODO: get rid of this
 
     @property
     def add_url(self) -> str:
         """Shortcut for the "add" URL of the model under test."""
-        return typing.cast(str, self.model.admin_add_url)  # type hinting for @classproperty doesn't work
+        return self.model.admin_add_url
 
     def assertBundle(  # pylint: disable=invalid-name
         self, cert: DjangoCAModelTypeVar, expected: Iterable[X509CertMixin], filename: str
@@ -1178,7 +1213,7 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
         self.assertEqual(response["Content-Disposition"], f"attachment; filename={filename}")
         self.assertEqual(response.content.decode("utf-8"), expected_content)
 
-    def assertCSS(self, response: HttpResponse, path: str) -> None:  # pylint: disable=invalid-name
+    def assertCSS(self, response: "HttpResponse", path: str) -> None:  # pylint: disable=invalid-name
         """Assert that the HTML from the given response includes the mentioned CSS."""
         if django.VERSION[:2] <= (4, 0):  # pragma: only django<4.1
             css = f'<link href="{static(path)}" type="text/css" media="all" rel="stylesheet" />'
@@ -1187,7 +1222,7 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
         self.assertInHTML(css, response.content.decode("utf-8"), 1)
 
     def assertChangeResponse(  # pylint: disable=invalid-name,unused-argument # obj is unused
-        self, response: HttpResponse, obj: DjangoCAModelTypeVar, status: int = HTTPStatus.OK
+        self, response: "HttpResponse", obj: DjangoCAModelTypeVar, status: int = HTTPStatus.OK
     ) -> None:
         """Assert that the passed response is a model change view."""
         self.assertEqual(response.status_code, status)
@@ -1199,7 +1234,7 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
             self.assertCSS(response, css)
 
     def assertChangelistResponse(  # pylint: disable=invalid-name
-        self, response: HttpResponse, *objects: models.Model, status: int = HTTPStatus.OK
+        self, response: "HttpResponse", *objects: models.Model, status: int = HTTPStatus.OK
     ) -> None:
         """Assert that the passed response is a model changelist view."""
         self.assertEqual(response.status_code, status)
@@ -1213,7 +1248,7 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
             self.assertCSS(response, css)
 
     def assertRequiresLogin(  # pylint: disable=invalid-name
-        self, response: HttpResponse, **kwargs: Any
+        self, response: "HttpResponse", **kwargs: Any
     ) -> None:
         """Assert that the given response is a redirect to the login page."""
         path = reverse("admin:login")
@@ -1223,12 +1258,12 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
     def change_url(self, obj: Optional[DjangoCAModel] = None) -> str:
         """Shortcut for the change URL of the given instance."""
         obj = obj or self.obj
-        return obj.admin_change_url  # type: ignore[union-attr]
+        return obj.admin_change_url
 
     @property
     def changelist_url(self) -> str:
         """Shortcut for the changelist URL of the model under test."""
-        return typing.cast(str, self.model.admin_changelist_url)
+        return self.model.admin_changelist_url
 
     def create_superuser(
         self, username: str = "admin", password: str = "admin", email: str = "user@example.com"
@@ -1239,19 +1274,19 @@ class AdminTestCaseMixin(TestCaseMixin, typing.Generic[DjangoCAModelTypeVar]):
     @contextmanager
     def freeze_time(
         self, timestamp: Union[str, datetime]
-    ) -> typing.Iterator[Union[FrozenDateTimeFactory, StepTickTimeFactory]]:
+    ) -> Iterator[Union[FrozenDateTimeFactory, StepTickTimeFactory]]:
         """Overridden to force a client login, otherwise the user session is expired."""
         with super().freeze_time(timestamp) as frozen:
             self.client.force_login(self.user)
             yield frozen
 
-    def get_changelist_view(self, data: Optional[Dict[str, str]] = None) -> HttpResponse:
+    def get_changelist_view(self, data: Optional[Dict[str, str]] = None) -> "HttpResponse":
         """Get the response to a changelist view for the given model."""
         return self.client.get(self.changelist_url, data)
 
     def get_change_view(
         self, obj: DjangoCAModelTypeVar, data: Optional[Dict[str, str]] = None
-    ) -> HttpResponse:
+    ) -> "HttpResponse":
         """Get the response to a change view for the given model instance."""
         return self.client.get(self.change_url(obj), data)
 
@@ -1272,12 +1307,12 @@ class StandardAdminViewTestCaseMixin(AdminTestCaseMixin[DjangoCAModelTypeVar]):
 
     def get_changelists(
         self,
-    ) -> typing.Iterator[Tuple[Iterable[DjangoCAModel], Dict[str, str]]]:
+    ) -> Iterator[Tuple[Iterable[DjangoCAModel], Dict[str, str]]]:
         """Generate list of objects for possible changelist views.
 
         Should yield tuples of objects that should be displayed and a dict of query parameters.
         """
-        yield (self.model.objects.all(), {})
+        yield self.model.objects.all(), {}
 
     def test_model_count(self) -> None:
         """Test that the implementing TestCase actually creates some instances."""
