@@ -1,28 +1,28 @@
-#!/usr/bin/env python3
-#
 # This file is part of django-ca (https://github.com/mathiasertl/django-ca).
 #
-# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
-# General Public License as published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 #
-# You should have received a copy of the GNU General Public License along with django-ca.  If not,
-# see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with django-ca. If not, see
+# <http://www.gnu.org/licenses/>.
 
 """The recreate-fixtures sub-command recreates the entire test fixture data.
 
 The test suite should be sufficiently modular to still run without errors after running this command."""
-
+import argparse
 import ipaddress
 import json
 import os
 import sys
 from datetime import datetime, timedelta
+from datetime import timezone as tz
 from pathlib import Path
+from typing import Any, Dict
 
 from cryptography import x509
 from cryptography.hazmat.primitives import hashes
@@ -35,15 +35,20 @@ DEFAULT_KEY_SIZE = 2048  # Size for private keys
 TIMEFORMAT = "%Y-%m-%d %H:%M:%S"
 ROOT_PATHLEN = None
 CHILD_PATHLEN = 0
-ECC_PATHLEN = 1
+EC_PATHLEN = 1
 PWD_PATHLEN = 2
 DSA_PATHLEN = 3
-DSA_ALGORITHM = hashes.SHA1()
 
 
 def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
-    dest, delay, only_contrib, regenerate_ocsp, generate_contrib, ca_validity, cert_validity
-):
+    dest: Path,
+    delay: bool,
+    only_contrib: bool,
+    regenerate_ocsp: bool,
+    generate_contrib: bool,
+    ca_validity: int,
+    cert_validity: int,
+) -> None:
     """Main entry function to recreate fixtures."""
     # pylint: disable=import-outside-toplevel  # django needs to be set up
     from django.core.management import call_command as manage
@@ -58,22 +63,28 @@ def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
         regenerate_ocsp_files,
     )
 
-    from django_ca.subject import Subject
+    from django_ca.utils import format_name, x509_name
 
     # pylint: enable=import-outside-toplevel
-
-    now = datetime.utcnow().replace(second=0, minute=0)
+    # The time-offsets from now from which CAs/certs are valid starts 25 days in the past, with the largest
+    # offset being 20 days. So the latest valid_from of any certs is five days in the past from when you run
+    # this script.
+    #
+    # This is important in tests, as the "everything_valid" timestamp is 23 days after the date computed
+    # below. If you use freezegun in a test and log in the setUp method (before freezegun freezes time),
+    # the session starts with the current, real time. Django ignores sessions that start in the future, so
+    # tests that use the test client would fail if "everything_valid" is in the future.
+    now = datetime.now(tz.utc).replace(second=0, minute=0, microsecond=0) - timedelta(days=25)
 
     manage("migrate", verbosity=0)
 
     out_path = dest / "cert-data.json"
     dest.mkdir(exist_ok=True)
 
-    data = {
+    data: Dict[str, Dict[str, Any]] = {
         "root": {
             "type": "ca",
-            "password": None,
-            "pathlen": ROOT_PATHLEN,
+            "path_length": ROOT_PATHLEN,
             "basic_constraints": {"critical": True, "value": {"ca": True}},
             "key_usage": "critical,cRLSign,keyCertSign",
         },
@@ -81,40 +92,64 @@ def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
             "type": "ca",
             "delta": timedelta(days=3),
             "parent": "root",
-            "password": None,
-            "basic_constraints": {"critical": True, "value": {"ca": True, "pathlen": CHILD_PATHLEN}},
-            "pathlen": CHILD_PATHLEN,
-            "max_pathlen": 0,
+            "basic_constraints": {"critical": True, "value": {"ca": True, "path_length": CHILD_PATHLEN}},
+            "path_length": CHILD_PATHLEN,
+            "max_path_length": 0,
         },
-        "ecc": {
+        "ec": {
             "type": "ca",
-            "password": None,
-            "basic_constraints": {"critical": True, "value": {"ca": True, "pathlen": ECC_PATHLEN}},
-            "pathlen": ECC_PATHLEN,
-            "key_size": 256,  # Value is unused in key generation, but needed for validation
-            "key_type": "ECC",
-            "max_pathlen": 1,
+            "basic_constraints": {"critical": True, "value": {"ca": True, "path_length": EC_PATHLEN}},
+            "path_length": EC_PATHLEN,
+            "key_type": "EC",
+            "max_path_length": 1,
         },
         "dsa": {
             "type": "ca",
-            "algorithm": DSA_ALGORITHM,
-            "password": None,
-            "basic_constraints": {"critical": True, "value": {"ca": True, "pathlen": DSA_PATHLEN}},
-            "pathlen": DSA_PATHLEN,
-            "max_pathlen": 3,
+            "key_type": "DSA",
+            "basic_constraints": {"critical": True, "value": {"ca": True, "path_length": DSA_PATHLEN}},
+            "path_length": DSA_PATHLEN,
+            "max_path_length": 3,
         },
         "pwd": {
             "type": "ca",
             "password": b"testpassword",
-            "basic_constraints": {"critical": True, "value": {"ca": True, "pathlen": PWD_PATHLEN}},
-            "pathlen": PWD_PATHLEN,
-            "max_pathlen": 2,
+            "basic_constraints": {"critical": True, "value": {"ca": True, "path_length": PWD_PATHLEN}},
+            "path_length": PWD_PATHLEN,
+            "max_path_length": 2,
         },
-        "root-cert": {"ca": "root", "delta": timedelta(days=5), "pathlen": ROOT_PATHLEN, "csr": True},
+        "ed25519": {
+            "type": "ca",
+            "key_type": "Ed25519",
+            "path_length": 1,
+            "max_path_length": 1,
+            "algorithm": None,
+        },
+        "ed448": {
+            "type": "ca",
+            "key_type": "Ed448",
+            "path_length": 1,
+            "max_path_length": 1,
+            "algorithm": None,
+        },
+        "root-cert": {"ca": "root", "delta": timedelta(days=5), "path_length": ROOT_PATHLEN, "csr": True},
         "child-cert": {"ca": "child", "delta": timedelta(days=5), "csr": True},
-        "ecc-cert": {"ca": "ecc", "delta": timedelta(days=5), "csr": True, "key_type": "ECC"},
+        "ec-cert": {"ca": "ec", "delta": timedelta(days=5), "csr": True, "key_type": "EC"},
         "pwd-cert": {"ca": "pwd", "delta": timedelta(days=5), "csr": True},
-        "dsa-cert": {"ca": "dsa", "delta": timedelta(days=5), "algorithm": DSA_ALGORITHM, "csr": True},
+        "dsa-cert": {"ca": "dsa", "delta": timedelta(days=5), "csr": True, "key_type": "DSA"},
+        "ed25519-cert": {
+            "ca": "ed25519",
+            "delta": timedelta(days=5),
+            "csr": True,
+            "algorithm": None,
+            "key_type": "Ed25519",
+        },
+        "ed448-cert": {
+            "ca": "ed448",
+            "delta": timedelta(days=5),
+            "csr": True,
+            "algorithm": None,
+            "key_type": "Ed448",
+        },
         "profile-client": {"ca": "child", "delta": timedelta(days=10), "csr": True},
         "profile-server": {"ca": "child", "delta": timedelta(days=10), "csr": True},
         "profile-webserver": {"ca": "child", "delta": timedelta(days=10), "csr": True},
@@ -125,15 +160,15 @@ def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
             "ca": "child",
             "delta": timedelta(days=20),
             "csr": True,
-            "subject": {
-                "C": "AT",
-                "ST": "Vienna",
-                "L": "Vienna",
-                "O": "Example",
-                "OU": "Example OU",
-                "CN": "all-extensions.example.com",
-                "emailAddress": "user@example.com",
-            },
+            "subject": [
+                ["C", "AT"],
+                ["ST", "Vienna"],
+                ["L", "Vienna"],
+                ["O", "Example"],
+                ["OU", "Example OU"],
+                ["CN", "all-extensions.example.com"],
+                ["emailAddress", "user@example.com"],
+            ],
             "extensions": {
                 "extended_key_usage": x509.Extension(
                     oid=ExtensionOID.EXTENDED_KEY_USAGE,
@@ -230,6 +265,15 @@ def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
             "ca": "child",
             "delta": timedelta(days=20),
             "csr": True,
+            "subject": [
+                ["C", "AT"],
+                ["ST", "Vienna"],
+                ["L", "Vienna"],
+                ["O", "Example"],
+                ["OU", "Example OU"],
+                ["CN", "alt-extensions.example.com"],
+                ["emailAddress", "user@example.com"],
+            ],
             "extensions": {
                 "authority_key_identifier": x509.Extension(
                     oid=ExtensionOID.AUTHORITY_KEY_IDENTIFIER,
@@ -342,25 +386,22 @@ def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
         },
     }
 
-    # Autocompute some values (name, filenames, ...) based on the dict key
+    # Auto-compute some values (name, filenames, ...) based on the dict key
     for cert_name, cert_values in data.items():
         cert_values["name"] = cert_name
         cert_values.setdefault("type", "cert")
         cert_values.setdefault("cat", "generated")
         cert_values.setdefault("algorithm", hashes.SHA256())
-        cert_values.setdefault("subject", {})
-        cert_values["subject"].setdefault("CN", f"{cert_name}.example.com")
-        cert_values["subject_str"] = str(Subject(cert_values["subject"]))
-        cert_values["csr_subject"] = {
-            k: f"csr.{v}" if k != "C" else v for k, v in cert_values["subject"].items()
-        }
-        cert_values["csr_subject_str"] = str(Subject(cert_values["csr_subject"]))
+        cert_values.setdefault("subject", [["CN", f"{cert_name}.example.com"]])
+        cert_values["subject_str"] = format_name(x509_name(cert_values["subject"]))
+        cert_values["csr_subject"] = {k: f"csr.{v}" if k != "C" else v for k, v in cert_values["subject"]}
         cert_values["key_filename"] = f"{cert_name}.key"
         cert_values["pub_filename"] = f"{cert_name}.pub"
         cert_values["key_der_filename"] = f"{cert_name}.key.der"
         cert_values["pub_der_filename"] = f"{cert_name}.pub.der"
-        cert_values.setdefault("key_size", DEFAULT_KEY_SIZE)
         cert_values.setdefault("key_type", "RSA")
+        if cert_values["key_type"] in ("RSA", "DSA"):
+            cert_values.setdefault("key_size", DEFAULT_KEY_SIZE)
         cert_values.setdefault("delta", timedelta())
         if cert_values.pop("csr", False):
             cert_values["csr_filename"] = f"{cert_name}.csr"
@@ -407,16 +448,16 @@ def recreate_fixtures(  # pylint: disable=too-many-locals,too-many-statements
             fixture_data = json.load(stream)
         fixture_data["certs"].update(data)
     else:
-        fixture_data = {"timestamp": now.strftime(TIMEFORMAT), "certs": data, "ocsp": ocsp_data}
+        fixture_data = {"timestamp": now.isoformat(), "certs": data, "ocsp": ocsp_data}
 
     with open(out_path, "w", encoding="utf-8") as stream:
-        json.dump(fixture_data, stream, indent=4, cls=CertificateEncoder)
+        json.dump(fixture_data, stream, indent=4, cls=CertificateEncoder, sort_keys=True)
 
 
 class Command(DevCommand):
     """Regenerate fixtures for testing."""
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
             "--only-contrib",
             default=False,
@@ -464,7 +505,7 @@ class Command(DevCommand):
             help="Where to store generated certificates (default: %(default)s).",
         )
 
-    def handle(self, args):
+    def handle(self, args: argparse.Namespace) -> None:
         if "TOX_ENV_DIR" in os.environ:  # was invoked via tox
             # insert ca/ into path, otherwise it won't find test_settings in django project
             sys.path.insert(0, str(config.SRC_DIR))
