@@ -1,17 +1,15 @@
-#!/usr/bin/env python3
-#
 # This file is part of django-ca (https://github.com/mathiasertl/django-ca).
 #
-# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU
-# General Public License as published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# django-ca is free software: you can redistribute it and/or modify it under the terms of the GNU General
+# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
+# option) any later version.
 #
-# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# django-ca is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 #
-# You should have received a copy of the GNU General Public License along with django-ca.  If not,
-# see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with django-ca. If not, see
+# <http://www.gnu.org/licenses/>.
 
 """The recreate-fixtures sub-command recreates the entire test fixture data.
 
@@ -24,7 +22,9 @@ import shutil
 import subprocess
 import tempfile
 from datetime import datetime
+from datetime import timezone as tz
 from pathlib import Path
+from typing import Any, Dict, List, Optional, Sequence, Union
 from unittest.mock import patch
 
 from cryptography import x509
@@ -33,12 +33,13 @@ from cryptography.hazmat.primitives.asymmetric import ec, rsa
 from cryptography.hazmat.primitives.serialization import (
     BestAvailableEncryption,
     Encoding,
+    KeySerializationEncryption,
     NoEncryption,
     PrivateFormat,
     load_pem_private_key,
 )
 from cryptography.x509 import ocsp
-from cryptography.x509.oid import ExtensionOID, NameOID
+from cryptography.x509.oid import NameOID
 
 from django.test.utils import override_settings
 from django.urls import reverse
@@ -47,56 +48,91 @@ from freezegun import freeze_time
 
 from devscripts import config, utils
 
-from django_ca import ca_settings
-from django_ca.extensions import OID_TO_EXTENSION, Extension
+from django_ca import ca_settings, constants
+from django_ca.extensions import serialize_extension
 from django_ca.models import Certificate, CertificateAuthority
 from django_ca.profiles import profiles
-from django_ca.subject import Subject
-from django_ca.utils import bytes_to_hex, ca_storage
+from django_ca.tests.base.typehints import CertFixtureData, OcspFixtureData
+from django_ca.typehints import ParsableKeyType
+from django_ca.utils import bytes_to_hex, ca_storage, format_name, serialize_name, x509_name
 
 DEFAULT_KEY_SIZE = 2048  # Size for private keys
 TIMEFORMAT = "%Y-%m-%d %H:%M:%S"
 
 
+def genpkey(*args: str) -> "subprocess.CompletedProcess[Any]":
+    """Convenience wrapper for the openssl genpkey program."""
+    return utils.run(["openssl", "genpkey"] + list(args), stderr=subprocess.DEVNULL)
+
+
 class CertificateEncoder(json.JSONEncoder):
     """Minor class to encode certificate data into json."""
 
-    def default(self, o):
+    def default(self, o: Any) -> Any:  # Any/Any matches base class typehints
         if isinstance(o, hashes.HashAlgorithm):
             return o.name
         if isinstance(o, Path):
             return str(o)
-
-        # Serializing extensions for now depends on django_ca.extension classes
         if isinstance(o, x509.Extension):
-            ext_class = OID_TO_EXTENSION[o.oid]
-            ext = ext_class(o)
-            return ext.serialize()
+            return serialize_extension(o)
         return json.JSONEncoder.default(self, o)
 
 
-def _create_key(path, key_type):
+def _create_key(path: Path, key_type: ParsableKeyType) -> None:
     if key_type == "RSA":
-        utils.run(["openssl", "genrsa", "-out", path, str(DEFAULT_KEY_SIZE)], stderr=subprocess.DEVNULL)
-    elif key_type == "ECC":
+        utils.run(["openssl", "genrsa", "-out", str(path), str(DEFAULT_KEY_SIZE)], stderr=subprocess.DEVNULL)
+    elif key_type == "DSA":
+        genpkey(
+            "-genparam",
+            "-algorithm",
+            "DSA",
+            "-out",
+            str(path.with_suffix(".param")),
+            "-pkeyopt",
+            "dsa_paramgen_bits:2048",
+            "-pkeyopt",
+            "dsa_paramgen_md:sha256",
+        )
+        genpkey("-paramfile", str(path.with_suffix(".param")), "-out", str(path))
+    elif key_type == "EC":
         utils.run(
-            ["openssl", "ecparam", "-name", "prime256v1", "-genkey", "-out", path],
+            ["openssl", "ecparam", "-name", "prime256v1", "-genkey", "-out", str(path)],
             stderr=subprocess.DEVNULL,
         )
+    elif key_type == "Ed25519":
+        genpkey("-algorithm", "ED25519", "-out", str(path))
+    elif key_type == "Ed448":
+        genpkey("-algorithm", "ED448", "-out", str(path))
     else:
         raise ValueError(f"Unknown key type: {key_type}")
 
 
-def _create_csr(key_path, path, subject="/CN=ignored.example.com", key_type="RSA"):
+def _create_csr(
+    key_path: Path, path: Path, subject: str = "/CN=ignored.example.com", key_type: ParsableKeyType = "RSA"
+) -> x509.CertificateSigningRequest:
     _create_key(key_path, key_type)
-    utils.run(["openssl", "req", "-new", "-key", key_path, "-out", path, "-utf8", "-batch", "-subj", subject])
+    utils.run(
+        [
+            "openssl",
+            "req",
+            "-new",
+            "-key",
+            str(key_path),
+            "-out",
+            str(path),
+            "-utf8",
+            "-batch",
+            "-subj",
+            subject,
+        ]
+    )
 
     with open(path, encoding="utf-8") as stream:
         csr = stream.read()
     return x509.load_pem_x509_csr(csr.encode("utf-8"))
 
 
-def _update_cert_data(cert, data):
+def _update_cert_data(cert: Union[CertificateAuthority, Certificate], data: Dict[str, Any]) -> None:
     data["serial"] = cert.serial
     data["hpkp"] = cert.hpkp_pin
     data["valid_from"] = cert.pub.loaded.not_valid_before.strftime(TIMEFORMAT)
@@ -107,47 +143,22 @@ def _update_cert_data(cert, data):
     data["sha256"] = cert.get_fingerprint(hashes.SHA256())
     data["sha512"] = cert.get_fingerprint(hashes.SHA512())
 
-    aki = cert.authority_key_identifier
-    if aki is not None:
-        data["authority_key_identifier"] = aki.serialize()
-
-    basic_constraints = cert.basic_constraints
-    if basic_constraints:
-        data["basic_constraints"] = basic_constraints.serialize()
-
-    ski = cert.subject_key_identifier
-    if ski is not None:
-        data["subject_key_identifier"] = ski.serialize()
-
-    key_usage = cert.key_usage
-    if key_usage is not None:
-        data["key_usage"] = key_usage.serialize()
-
-    aia = cert.authority_information_access
-    if aia is not None:
-        data["authority_information_access"] = aia.serialize()
-
-    san = cert.subject_alternative_name
-    if san is not None:
-        data["subject_alternative_name"] = san.serialize()
-
-    ian = cert.issuer_alternative_name
-    if ian is not None:
-        data["issuer_alternative_name"] = ian.serialize()
-
-    eku = cert.extended_key_usage
-    if eku is not None:
-        data["extended_key_usage"] = eku.serialize()
-    crldp = cert.crl_distribution_points
-    if crldp is not None:
-        data["crl_distribution_points"] = crldp.serialize()
+    for oid, ext in cert.x509_extensions.items():
+        ext_key = constants.EXTENSION_KEYS[oid]
+        data[ext_key] = serialize_extension(ext)
 
 
-def _write_ca(dest, ca, cert_data, testserver, password=None):
-    key_dest = os.path.join(dest, cert_data["key_filename"])
-    pub_dest = os.path.join(dest, cert_data["pub_filename"])
-    key_der_dest = os.path.join(dest, cert_data["key_der_filename"])
-    pub_der_dest = os.path.join(dest, cert_data["pub_der_filename"])
+def _write_ca(
+    dest: Path,
+    ca: CertificateAuthority,
+    cert_data: CertFixtureData,
+    testserver: str,
+    password: Optional[bytes] = None,
+) -> None:
+    key_dest = dest / cert_data["key_filename"]
+    pub_dest = dest / cert_data["pub_filename"]
+    key_der_dest = dest / cert_data["key_der_filename"]
+    pub_der_dest = dest / cert_data["pub_der_filename"]
 
     # write files to dest
     shutil.copy(ca_storage.path(ca.private_key_path), key_dest)
@@ -155,7 +166,7 @@ def _write_ca(dest, ca, cert_data, testserver, password=None):
         stream.write(ca.pub.pem)
 
     if password is None:
-        encryption = NoEncryption()
+        encryption: KeySerializationEncryption = NoEncryption()
     else:
         encryption = BestAvailableEncryption(password)
 
@@ -179,12 +190,18 @@ def _write_ca(dest, ca, cert_data, testserver, password=None):
     _update_cert_data(ca, cert_data)
 
 
-def _copy_cert(dest, cert, data, key_path, csr_path):
-    key_dest = os.path.join(dest, data["key_filename"])
-    csr_dest = os.path.join(dest, data["csr_filename"])
-    pub_dest = os.path.join(dest, data["pub_filename"])
-    key_der_dest = os.path.join(dest, data["key_der_filename"])
-    pub_der_dest = os.path.join(dest, data["pub_der_filename"])
+def _copy_cert(
+    dest: Path,
+    cert: Certificate,
+    data: CertFixtureData,
+    key_path: Path,
+    csr_path: Path,
+) -> None:
+    key_dest = dest / data["key_filename"]
+    csr_dest = dest / data["csr_filename"]
+    pub_dest = dest / data["pub_filename"]
+    key_der_dest = dest / data["key_der_filename"]
+    pub_der_dest = dest / data["pub_der_filename"]
 
     shutil.copy(key_path, key_dest)
     shutil.copy(csr_path, csr_dest)
@@ -193,8 +210,8 @@ def _copy_cert(dest, cert, data, key_path, csr_path):
 
     with open(key_dest, "rb") as stream:
         priv_key = stream.read()
-    priv_key = load_pem_private_key(priv_key, None)
-    key_der = priv_key.private_bytes(
+    loaded_priv_key = load_pem_private_key(priv_key, None)
+    key_der = loaded_priv_key.private_bytes(
         encoding=Encoding.DER, format=PrivateFormat.PKCS8, encryption_algorithm=NoEncryption()
     )
     with open(key_der_dest, "wb") as stream:
@@ -203,13 +220,20 @@ def _copy_cert(dest, cert, data, key_path, csr_path):
         stream.write(cert.pub.der)
 
     data["crl"] = cert.ca.crl_url
-    data["subject"] = cert.distinguished_name
+    data["subject"] = serialize_name(cert.subject)
+    data["subject_str"] = format_name(cert.subject)
     data["parsed_cert"] = cert
 
     _update_cert_data(cert, data)
 
 
-def _update_contrib(parsed, data, cert, name, filename):
+def _update_contrib(
+    parsed: x509.Certificate,
+    data: Dict[str, Any],
+    cert: Union[Certificate, CertificateAuthority],
+    name: str,
+    filename: str,
+) -> None:
     cert_data = {
         "name": name,
         "cn": cert.cn,
@@ -220,7 +244,8 @@ def _update_contrib(parsed, data, cert, name, filename):
         "valid_from": parsed.not_valid_before.strftime(TIMEFORMAT),
         "valid_until": parsed.not_valid_after.strftime(TIMEFORMAT),
         "serial": cert.serial,
-        "subject": cert.distinguished_name,
+        "subject": serialize_name(cert.subject),
+        "subject_str": format_name(cert.subject),
         "hpkp": cert.hpkp_pin,
         "md5": cert.get_fingerprint(hashes.MD5()),
         "sha1": cert.get_fingerprint(hashes.SHA1()),
@@ -228,27 +253,18 @@ def _update_contrib(parsed, data, cert, name, filename):
         "sha512": cert.get_fingerprint(hashes.SHA512()),
     }
 
-    for ext in cert.extensions:
-        if isinstance(ext, Extension):
-            key = OID_TO_EXTENSION[ext.oid].key
-            cert_data[key] = ext.serialize()
-        elif isinstance(ext, tuple):
-            print("### get extension tuple!!!")
-            key, value = ext
-            if isinstance(value[1], x509.ObjectIdentifier):
-                # Currently just some old StartSSL extensions for Netscape (!)
-                continue
-            cert_data[key] = value
+    for oid, ext in cert.x509_extensions.items():
+        if isinstance(ext.value, x509.UnrecognizedExtension):
+            # Currently just some old StartSSL extensions for Netscape (!)
+            continue
 
-    try:
-        ext = cert.pub.loaded.extensions.get_extension_for_oid(ExtensionOID.CERTIFICATE_POLICIES).value
-    except x509.ExtensionNotFound:
-        pass
+        ext_key = constants.EXTENSION_KEYS[oid]
+        cert_data[ext_key] = serialize_extension(ext)
 
     data[name] = cert_data
 
 
-def _generate_contrib_files(data):
+def _generate_contrib_files(data: Dict[str, Dict[str, Any]]) -> None:
     files_dir = config.DOCS_DIR / "source" / "_files"
     for filename in (files_dir / "ca").iterdir():
         name = filename.stem
@@ -260,15 +276,15 @@ def _generate_contrib_files(data):
         ca = CertificateAuthority(name=name)
         ca.update_certificate(parsed)
 
-        _update_contrib(parsed, data, ca, name, filename)
+        _update_contrib(parsed, data, ca, name, filename.name)
         data[name]["type"] = "ca"
-        data[name]["pathlen"] = ca.pathlen
+        data[name]["path_length"] = ca.path_length
 
         public_key = parsed.public_key()
         if isinstance(public_key, rsa.RSAPublicKey):
             data[name]["key_type"] = "RSA"
         elif isinstance(public_key, ec.EllipticCurvePublicKey):
-            data[name]["key_type"] = "ECC"
+            data[name]["key_type"] = "EC"
         else:
             raise ValueError(f"Unknown type of Public key encountered: {public_key}")
 
@@ -287,7 +303,7 @@ def _generate_contrib_files(data):
         parsed = x509.load_pem_x509_certificate(pem)
         cert = Certificate()
         cert.update_certificate(parsed)
-        _update_contrib(parsed, data, cert, name, filename)
+        _update_contrib(parsed, data, cert, name, filename.name)
         data[name]["type"] = "cert"
 
         if contrib_ca:
@@ -297,31 +313,31 @@ def _generate_contrib_files(data):
         if isinstance(public_key, rsa.RSAPublicKey):
             data[name]["key_type"] = "RSA"
         elif isinstance(public_key, ec.EllipticCurvePublicKey):
-            data[name]["key_type"] = "ECC"
+            data[name]["key_type"] = "EC"
         else:
             raise ValueError(f"Unknown type of Public key encountered: {public_key}")
 
 
-def create_cas(dest, now, delay, data):
+def create_cas(dest: Path, now: datetime, delay: bool, data: CertFixtureData) -> List[CertificateAuthority]:
     """Create CAs."""
     testserver = f"http://{ca_settings.CA_DEFAULT_HOSTNAME}"
     ca_names = [v["name"] for k, v in data.items() if v.get("type") == "ca"]
 
     # sort ca_names so that any children are created last
-    ca_names = sorted(ca_names, key=lambda n: data[n].get("parent", ""))
+    ca_names = sorted(ca_names, key=lambda n: data[n].get("parent", ""))  # type: ignore[no-any-return]
     ca_instances = []
 
     for name in ca_names:
-        kwargs = {}
-
         # Get some data from the parent, if present
-        parent = data[name].get("parent")
-        if parent:
-            kwargs["parent"] = CertificateAuthority.objects.get(name=parent)
-            kwargs["ca_crl_url"] = [data[parent]["ca_crl_url"]]
+        parent: Optional[CertificateAuthority] = None
+        ca_crl_url: Optional[Sequence[str]] = None
+        parent_name = data[name].get("parent")
+        if parent_name:
+            parent = CertificateAuthority.objects.get(name=parent_name)
+            ca_crl_url = [data[parent_name]["ca_crl_url"]]
 
             # also update data
-            data[name]["crl"] = data[parent]["ca_crl_url"]
+            data[name]["crl"] = data[parent_name]["ca_crl_url"]
 
         freeze_now = now
         if delay:
@@ -330,14 +346,15 @@ def create_cas(dest, now, delay, data):
         with freeze_time(freeze_now):
             ca = CertificateAuthority.objects.init(
                 name=data[name]["name"],
-                password=data[name]["password"],
-                subject=Subject(data[name]["subject"]).name,
-                expires=datetime.utcnow() + data[name]["expires"],
+                password=data[name].get("password"),
+                subject=x509_name(data[name]["subject"]),
+                expires=datetime.now(tz=tz.utc) + data[name]["expires"],
                 key_type=data[name]["key_type"],
-                key_size=data[name]["key_size"],
-                algorithm=data[name]["algorithm"],
-                pathlen=data[name]["pathlen"],
-                **kwargs,
+                key_size=data[name].get("key_size"),
+                algorithm=data[name].get("algorithm"),
+                path_length=data[name]["path_length"],
+                parent=parent,
+                ca_crl_url=ca_crl_url,
             )
 
         # Same values can only be added here because they require data from the already created CA
@@ -346,24 +363,27 @@ def create_cas(dest, now, delay, data):
         ca.save()
 
         ca_instances.append(ca)
-        _write_ca(dest, ca, data[name], testserver, password=data[name]["password"])
+        _write_ca(dest, ca, data[name], testserver, password=data[name].get("password"))
 
     # add parent/child relationships
     data["root"]["children"] = [[data["child"]["name"], data["child"]["serial"]]]
     return ca_instances
 
 
-def create_certs(dest, cas, now, delay, data):
+def create_certs(
+    dest: Path, cas: Sequence[CertificateAuthority], now: datetime, delay: bool, data: Dict[str, Any]
+) -> None:
     """Create regular certificates."""
     # let's create a standard certificate for every CA
     for ca in cas:
         name = f"{ca.name}-cert"
-        key_path = os.path.join(ca_settings.CA_DIR, f"{name}.key")
-        csr_path = os.path.join(ca_settings.CA_DIR, f"{name}.csr")
+        key_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.key"))
+        csr_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.csr"))
+        csr_subject = format_name(x509_name(data[name]["csr_subject"].items()))
         csr = _create_csr(
             key_path,
             csr_path,
-            subject=data[name]["csr_subject_str"],
+            subject=csr_subject,
             key_type=data[name]["key_type"],
         )
 
@@ -371,8 +391,7 @@ def create_certs(dest, cas, now, delay, data):
         if delay:
             freeze_now += data[name]["delta"]
 
-        pwd = data[data[name]["ca"]]["password"]
-        subject = Subject(f"/CN={data[name]['cn']}")
+        pwd = data[data[name]["ca"]].get("password")
         with freeze_time(freeze_now):
             cert = Certificate.objects.create_cert(
                 ca=ca,
@@ -381,7 +400,7 @@ def create_certs(dest, cas, now, delay, data):
                 expires=data[name]["expires"],
                 algorithm=data[name]["algorithm"],
                 password=pwd,
-                subject=subject,
+                subject=x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, data[name]["cn"])]),
             )
         _copy_cert(dest, cert, data[name], key_path, csr_path)
 
@@ -390,12 +409,13 @@ def create_certs(dest, cas, now, delay, data):
         name = f"profile-{profile}"
         ca = CertificateAuthority.objects.get(name=data[name]["ca"])
 
-        key_path = os.path.join(ca_settings.CA_DIR, f"{name}.key")
-        csr_path = os.path.join(ca_settings.CA_DIR, f"{name}.csr")
+        key_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.key"))
+        csr_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.csr"))
+        csr_subject = format_name(x509_name(data[name]["csr_subject"].items()))
         csr = _create_csr(
             key_path,
             csr_path,
-            subject=data[name]["csr_subject_str"],
+            subject=csr_subject,
             key_type=data[name]["key_type"],
         )
 
@@ -403,8 +423,7 @@ def create_certs(dest, cas, now, delay, data):
         if delay:
             freeze_now += data[name]["delta"]
 
-        pwd = data[ca.name]["password"]
-        subject = Subject(f"/CN={data[name]['cn']}")
+        pwd = data[ca.name].get("password")
         with freeze_time(freeze_now):
             cert = Certificate.objects.create_cert(
                 ca=ca,
@@ -413,28 +432,29 @@ def create_certs(dest, cas, now, delay, data):
                 algorithm=data[name]["algorithm"],
                 expires=data[name]["expires"],
                 password=pwd,
-                subject=subject,
+                subject=x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, data[name]["cn"])]),
             )
 
         data[name]["profile"] = profile
         _copy_cert(dest, cert, data[name], key_path, csr_path)
 
 
-def create_special_certs(dest, now, delay, data):
+def create_special_certs(dest: Path, now: datetime, delay: bool, data: CertFixtureData) -> None:
     """Create special-interest certificates (edge cases etc.)."""
     # create a cert with absolutely no extensions
     name = "no-extensions"
     ca = CertificateAuthority.objects.get(name=data[name]["ca"])
-    key_path = os.path.join(ca_settings.CA_DIR, f"{name}.key")
-    csr_path = os.path.join(ca_settings.CA_DIR, f"{name}.csr")
-    csr = _create_csr(key_path, csr_path, subject=data[name]["csr_subject_str"])
+    key_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.key"))
+    csr_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.csr"))
+    csr_subject = format_name(x509_name(data[name]["csr_subject"].items()))
+    csr = _create_csr(key_path, csr_path, subject=csr_subject)
 
     freeze_now = now
     if delay:
         freeze_now += data[name]["delta"]
     with freeze_time(freeze_now):
         no_ext_now = datetime.utcnow()
-        pwd = data[ca.name]["password"]
+        pwd = data[ca.name].get("password")
         subject = x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, data[name]["cn"])])
 
         builder = x509.CertificateBuilder()
@@ -465,21 +485,21 @@ def create_special_certs(dest, now, delay, data):
     #       https://github.com/pyca/cryptography/issues/1947)
     name = "all-extensions"
     ca = CertificateAuthority.objects.get(name=data[name]["ca"])
-    pwd = data[ca.name]["password"]
-    key_path = os.path.join(ca_settings.CA_DIR, f"{name}.key")
-    csr_path = os.path.join(ca_settings.CA_DIR, f"{name}.csr")
-    csr = _create_csr(key_path, csr_path, subject=data[name]["csr_subject_str"])
+    key_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.key"))
+    csr_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.csr"))
+    csr_subject = format_name(x509_name(data[name]["csr_subject"].items()))
+    csr = _create_csr(key_path, csr_path, subject=csr_subject)
 
     with freeze_time(now + data[name]["delta"]):
         cert = Certificate.objects.create_cert(
             ca=ca,
             csr=csr,
             profile=profiles["webserver"],
-            algorithm=data[name]["algorithm"],
-            subject=data[name]["subject"],
+            algorithm=data[name].get("algorithm"),
+            subject=x509_name(data[name]["subject"]),
             expires=data[name]["expires"],
-            password=pwd,
-            extensions=data[name]["extensions"],
+            password=data[ca.name].get("password"),
+            extensions=data[name]["extensions"].values(),
         )
     data[name].update(data[name].pop("extensions"))  # cert_data expects this to be flat
     _copy_cert(dest, cert, data[name], key_path, csr_path)
@@ -489,33 +509,33 @@ def create_special_certs(dest, now, delay, data):
     name = "alt-extensions"
     ca = CertificateAuthority.objects.get(name=data[name]["ca"])
     ca.crl_url = ""
-    pwd = data[ca.name]["password"]
-    key_path = os.path.join(ca_settings.CA_DIR, f"{name}.key")
-    csr_path = os.path.join(ca_settings.CA_DIR, f"{name}.csr")
-    csr = _create_csr(key_path, csr_path, subject=data[name]["csr_subject_str"])
+    key_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.key"))
+    csr_path = Path(os.path.join(ca_settings.CA_DIR, f"{name}.csr"))
+    csr_subject = format_name(x509_name(data[name]["csr_subject"].items()))
+    csr = _create_csr(key_path, csr_path, subject=csr_subject)
 
     with freeze_time(now + data[name]["delta"]):
         cert = Certificate.objects.create_cert(
             ca=ca,
             csr=csr,
             profile=profiles["webserver"],
-            algorithm=data[name]["algorithm"],
-            subject=data[name]["subject"],
+            algorithm=data[name].get("algorithm"),
+            subject=x509_name(data[name]["subject"]),
             expires=data[name]["expires"],
-            password=pwd,
-            extensions=data[name]["extensions"],
+            password=data[ca.name].get("password"),
+            extensions=data[name]["extensions"].values(),
         )
     data[name].update(data[name].pop("extensions"))  # cert_data expects this to be flat
     _copy_cert(dest, cert, data[name], key_path, csr_path)
 
 
-def regenerate_ocsp_files(dest, data):
+def regenerate_ocsp_files(dest: Path, data: CertFixtureData) -> Dict[str, OcspFixtureData]:
     """Regenerate OCSP example requests."""
-    ocsp_data = {
+    ocsp_data: Dict[str, OcspFixtureData] = {
         "nonce": {"name": "nonce", "filename": "nonce.req"},
         "no-nonce": {"name": "no-nonce", "filename": "no-nonce.req"},
     }
-    ocsp_base = os.path.join(dest, "ocsp")
+    ocsp_base = dest / "ocsp"
     if not os.path.exists(ocsp_base):
         os.makedirs(ocsp_base)
     ocsp_builder = ocsp.OCSPRequestBuilder()
@@ -526,14 +546,14 @@ def regenerate_ocsp_files(dest, data):
     )
 
     no_nonce_req = ocsp_builder.build().public_bytes(Encoding.DER)
-    with open(os.path.join(ocsp_base, ocsp_data["no-nonce"]["filename"]), "wb") as stream:
+    with open(ocsp_base / ocsp_data["no-nonce"]["filename"], "wb") as stream:
         stream.write(no_nonce_req)
 
     nonce = os.urandom(16)
     ocsp_data["nonce"]["nonce"] = bytes_to_hex(nonce)
     ocsp_builder = ocsp_builder.add_extension(x509.OCSPNonce(nonce), critical=False)
     nonce_req = ocsp_builder.build().public_bytes(Encoding.DER)
-    with open(os.path.join(ocsp_base, ocsp_data["nonce"]["filename"]), "wb") as stream:
+    with open(ocsp_base / ocsp_data["nonce"]["filename"], "wb") as stream:
         stream.write(nonce_req)
     return ocsp_data
 
@@ -541,7 +561,7 @@ def regenerate_ocsp_files(dest, data):
 class override_tmpcadir(override_settings):  # pylint: disable=invalid-name
     """Simplified copy of the same decorator in tests.base."""
 
-    def enable(self):
+    def enable(self) -> None:
         # pylint: disable=attribute-defined-outside-init
         self.options["CA_DIR"] = tempfile.mkdtemp()
         self.mock = patch.object(ca_storage, "location", self.options["CA_DIR"])
@@ -554,7 +574,7 @@ class override_tmpcadir(override_settings):  # pylint: disable=invalid-name
         self.mockc = patch.object(ca_settings, "CA_DIR", self.options["CA_DIR"])
         self.mockc.start()
 
-    def disable(self):
+    def disable(self) -> None:
         super().disable()
         self.mock.stop()
         self.mock_.stop()
